// Copyright 2002-2012, University of Colorado
/**
 * Main application class for PhET's Energy Forms and Changes simulation.
 *
 * @author Sam Reid
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var ActionEvent = require( 'java.awt.event.ActionEvent' );
  var ActionListener = require( 'java.awt.event.ActionListener' );
  var WindowAdapter = require( 'java.awt.event.WindowAdapter' );
  var WindowEvent = require( 'java.awt.event.WindowEvent' );
  var swing = require( 'javax.swing' );//.*
  var PhetApplicationConfig = require( 'edu.colorado.phet.common.phetcommon.application.PhetApplicationConfig' );
  var PhetApplicationLauncher = require( 'edu.colorado.phet.common.phetcommon.application.PhetApplicationLauncher' );
  var PhetFrame = require( 'edu.colorado.phet.common.phetcommon.view.PhetFrame' );
  var PiccoloPhetApplication = require( 'edu.colorado.phet.common.piccolophet.PiccoloPhetApplication' );
  var EnergySystemsModule = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/EnergySystemsModule' );
  var EFACIntroModule = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/EFACIntroModule' );
  var HeatTransferValuesDialog = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/dev/HeatTransferValuesDialog' );
  var EFACIntroCanvas = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/view/EFACIntroCanvas' );

  function EnergyFormsAndChangesApplication( config ) {

    //private
    this.showHeatTransferValuesDialogCheckBox;

    //private
    this.heatTransferValuesDialog = null;
    PiccoloPhetApplication.call( this, config );
    // Create the modules
    addModule( new EFACIntroModule() );
    addModule( new EnergySystemsModule() );
    initMenuBar();
  }

  return inherit( PiccoloPhetApplication, EnergyFormsAndChangesApplication, {

    //private
    initMenuBar: function() {
      var frame = getPhetFrame();
      // Developer menu
      var developerMenu = frame.getDeveloperMenu();
      // Add a check box for controlling whether the button for dumping energy values is visible.
      var showDumpEnergiesButtonCheckBox = new JCheckBoxMenuItem( "Show Dump Energies Button" ).withAnonymousClassBody( {
        initializer: function() {
          setSelected( EFACIntroCanvas.showDumpEnergiesButton.get() );
          addActionListener( new ActionListener().withAnonymousClassBody( {
            actionPerformed: function( e ) {
              EFACIntroCanvas.showDumpEnergiesButton.set( isSelected() );
            }
          } ) );
        }
      } );
      // Add a check box for controlling whether the button for dumping energy values is visible.
      showHeatTransferValuesDialogCheckBox = new JCheckBoxMenuItem( "Show Heat Transfer Values Dialog" ).withAnonymousClassBody( {
        initializer: function() {
          addActionListener( new ActionListener().withAnonymousClassBody( {
            actionPerformed: function( e ) {
              setHeatTransferValuesDialogVisible( isSelected() );
            }
          } ) );
        }
      } );
      developerMenu.add( showDumpEnergiesButtonCheckBox );
      developerMenu.add( showHeatTransferValuesDialogCheckBox );
    },

    //private
    setHeatTransferValuesDialogVisible: function( isVisible ) {
      if ( isVisible && heatTransferValuesDialog == null ) {
        // Dialog hasn't been created yet, so create it.
        heatTransferValuesDialog = new HeatTransferValuesDialog( getPhetFrame() );
        // Just hide when closed so its position is retained.
        heatTransferValuesDialog.setDefaultCloseOperation( JFrame.HIDE_ON_CLOSE );
        // position when moved after that).
        heatTransferValuesDialog.setLocationRelativeTo( null );
        // dialog itself.
        heatTransferValuesDialog.addWindowListener( new WindowAdapter().withAnonymousClassBody( {
          windowClosing: function( e ) {
            showHeatTransferValuesDialogCheckBox.setSelected( false );
          }
        } ) );
      }
      if ( heatTransferValuesDialog != null ) {
        heatTransferValuesDialog.setVisible( isVisible );
      }
    },
    main: function( args ) {
      new PhetApplicationLauncher().launchSim( args, EnergyFormsAndChangesResources.PROJECT_NAME, "energy-forms-and-changes", EnergyFormsAndChangesApplication.class );
    }
  } );
} );

