// Copyright 2002-2012, University of Colorado
/**
 * Basic thermometer that senses temperature, has a position. The thermometer
 * has only a point and a temperature in the model.  Its visual representation
 * is left entirely to the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Vector2 = require( 'DOT/Vector2' );
  var ClockAdapter = require( 'edu.colorado.phet.common.phetcommon.model.clock.ClockAdapter' );
  var ClockEvent = require( 'edu.colorado.phet.common.phetcommon.model.clock.ClockEvent' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var PhetColorScheme = require( 'edu.colorado.phet.common.phetcommon.view.PhetColorScheme' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var HorizontalSurface = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/HorizontalSurface' );
  var TemperatureAndColor = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/TemperatureAndColor' );
  var UserMovableModelElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/UserMovableModelElement' );

  /**
   * Constructor.
   */
  function Thermometer( clock, model, initialPosition, initiallyActive ) {
    this.sensedTemperature = new Property( EFACConstants.ROOM_TEMPERATURE );
    this.sensedElementColor = new Property( PhetColorScheme.RED_COLORBLIND );
    // Property that is used primarily to control visibility in the view.
    this.active;
    UserMovableModelElement.call( this, initialPosition );
    active = new BooleanProperty( initiallyActive );
    // Update the sensed temperature at each clock tick.
    clock.addClockListener( new ClockAdapter().withAnonymousClassBody( {
      clockTicked: function( clockEvent ) {
        var temperatureAndColor = model.getTemperatureAndColorAtLocation( position.get() );
        sensedTemperature.set( temperatureAndColor.temperature );
        sensedElementColor.set( temperatureAndColor.color );
      }
    } ) );
  }

  return inherit( UserMovableModelElement, Thermometer, {
    reset: function() {
      active.reset();
    },
    getUserComponent: function() {
      return EnergyFormsAndChangesSimSharing.UserComponents.thermometer;
    },
    getBottomSurfaceProperty: function() {
      // Doesn't have a bottom surface, and can't be set on anything.
      return null;
    }
  } );
} );

