// Copyright 2002-2012, University of Colorado
/**
 * This class is a PNode that monitors the comings and goings of energy
 * chunks on a observable list and adds/removes them from this node.  This is
 * intended to be used in other PNodes that represent model elements that
 * contain energy chunks.
 * <p/>
 * This was done as a separate class so that it could be used in composition
 * rather than inheritance, because composition allows better control over the
 * layering within the parent PNode.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'DOT/Vector2' );
  var Property = require( 'AXON/Property' );
  var ObservableArray = require( 'AXON/ObservableArray' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );
  var Node = require( 'SCENERY/nodes/Node' );

  function EnergyChunkLayer( energyChunkList, parentPosition, mvt ) {
    // existence in the model.
    energyChunkList.addElementAddedObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( addedEnergyChunk ) {
        var energyChunkNode = new EnergyChunkNode( addedEnergyChunk, mvt );
        addChild( energyChunkNode );
        // Remove the energy chunk nodes as they are removed from the model.
        energyChunkList.addElementRemovedObserver( new VoidFunction1().withAnonymousClassBody( {
          apply: function( removedEnergyChunk ) {
            if ( removedEnergyChunk == addedEnergyChunk ) {
              removeChild( energyChunkNode );
            }
          }
        } ) );
      }
    } ) );
    // compensate.
    parentPosition.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( position ) {
        setOffset( -mvt.modelToViewX( position.x ), -mvt.modelToViewY( position.y ) );
      }
    } ) );
  }

  return inherit( Node, EnergyChunkLayer, {
  } );
} );

