// Copyright 2002-2012, University of Colorado
/**
 * Base class for model elements that use energy, and generally do something
 * as a result.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var Random = require( 'java.util.Random' );
  var Vector2 = require( 'DOT/Vector2' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var ObservableArray = require( 'AXON/ObservableArray' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var Beaker = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Beaker' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );
  var ITemperatureModel = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/ITemperatureModel' );
  var Thermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Thermometer' );
  var HeatTransferConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/HeatTransferConstants' );
  var TemperatureAndColor = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/TemperatureAndColor' );
  var ELEMENT_BASE_BACK = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/ELEMENT_BASE_BACK' );//static
  var ELEMENT_BASE_FRONT = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/ELEMENT_BASE_FRONT' );//static
  var HEATER_ELEMENT = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/HEATER_ELEMENT' );//static
  var HEATER_ELEMENT_DARK = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/HEATER_ELEMENT_DARK' );//static
  var WIRE_BLACK_62 = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/WIRE_BLACK_62' );//static
  var WIRE_BLACK_RIGHT = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources/Images/WIRE_BLACK_RIGHT' );//static
  var ROOM_TEMPERATURE = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants/ROOM_TEMPERATURE' );//static

//-------------------------------------------------------------------------
// Class Data
//-------------------------------------------------------------------------
// Images used to represent this class in the view.

  //private
  var HEATER_ELEMENT_OFFSET = new Vector2( -0.002, 0.022 );
  var WIRE_STRAIGHT_IMAGE = new ModelElementImage( WIRE_BLACK_62, new Vector2( -0.036, -0.04 ) );
  var WIRE_CURVE_IMAGE = new ModelElementImage( WIRE_BLACK_RIGHT, new Vector2( -0.009, -0.016 ) );
  var ELEMENT_BASE_BACK_IMAGE = new ModelElementImage( ELEMENT_BASE_BACK, new Vector2( 0, 0 ) );
  var ELEMENT_BASE_FRONT_IMAGE = new ModelElementImage( ELEMENT_BASE_FRONT, new Vector2( 0, 0.0005 ) );
  var HEATER_ELEMENT_OFF_IMAGE = new ModelElementImage( HEATER_ELEMENT_DARK, HEATER_ELEMENT_OFFSET );
  var HEATER_ELEMENT_ON_IMAGE = new ModelElementImage( HEATER_ELEMENT, HEATER_ELEMENT_OFFSET );
// Offsets need for creating the path followed by the energy chunks.  These
// were empirically determined based on images, will need to change if the
// images are changed.

  //private
  var OFFSET_TO_LEFT_SIDE_OF_WIRE = new Vector2( -0.04, -0.04 );

  //private
  var OFFSET_TO_LEFT_SIDE_OF_WIRE_BEND = new Vector2( -0.02, -0.04 );

  //private
  var OFFSET_TO_FIRST_WIRE_CURVE_POINT = new Vector2( -0.01, -0.0375 );

  //private
  var OFFSET_TO_SECOND_WIRE_CURVE_POINT = new Vector2( -0.001, -0.025 );

  //private
  var OFFSET_TO_THIRD_WIRE_CURVE_POINT = new Vector2( -0.0005, -0.0175 );

  //private
  var OFFSET_TO_BOTTOM_OF_CONNECTOR = new Vector2( 0, -0.01 );

  //private
  var OFFSET_TO_CONVERSION_POINT = new Vector2( 0, 0.012 );
// Miscellaneous other constants.

  //private
  var RAND = new Random();
// In meters.

  //private
  var BEAKER_WIDTH = 0.075;

  //private
  var BEAKER_HEIGHT = BEAKER_WIDTH * 0.9;

  //private
  var BEAKER_OFFSET = new Vector2( 0, 0.025 );

  //private
  var THERMOMETER_OFFSET = new Vector2( 0.033, 0.035 );
// In meters/sec, quite slow.

  //private
  var HEATING_ELEMENT_ENERGY_CHUNK_VELOCITY = 0.0075;

  //private
  var HEATER_ELEMENT_2D_HEIGHT = HEATER_ELEMENT_OFF_IMAGE.getHeight();
// Joules/sec, not connected to incoming energy.

  //private
  var MAX_HEAT_GENERATION_RATE = 5000;
// In meters.

  //private
  var RADIATED_ENERGY_CHUNK_TRAVEL_DISTANCE = 0.2;
// In proportion per second.

  //private
  var HEAT_ENERGY_CHANGE_RATE = 0.5;

  function BeakerHeater( clock, energyChunksVisible ) {
    //-------------------------------------------------------------------------
    // Instance Data
    //-------------------------------------------------------------------------
    // Proportion, from 0 to 1, of the max possible heat being generated by heater element.
    this.heatProportion = new Property( 0.0 );

    //private
    this.electricalEnergyChunkMovers = [];

    //private
    this.heatingElementEnergyChunkMovers = [];

    //private
    this.radiatedEnergyChunkMovers = [];
    this.beaker;
    this.thermometer;

    //private
    this.energyChunksVisible;
    // Separate list for the radiated energy chunks so that they can be
    // handled a little differently in the view.
    this.radiatedEnergyChunkList = new ObservableArray();
    EnergyUser.call( this, EnergyFormsAndChangesResources.Images.WATER_ICON );
    this.energyChunksVisible = energyChunksVisible;
    beaker = new Beaker( clock, BEAKER_OFFSET, BEAKER_WIDTH, BEAKER_HEIGHT, energyChunksVisible );
    thermometer = new Thermometer( clock, new ITemperatureModel().withAnonymousClassBody( {
      getTemperatureAndColorAtLocation: function( location ) {
        return new TemperatureAndColor( beaker.getTemperature(), EFACConstants.WATER_COLOR_OPAQUE );
      }
    } ), THERMOMETER_OFFSET, true );
    // model element position changes.
    getObservablePosition().addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( position ) {
        beaker.position.set( position.plus( BEAKER_OFFSET ) );
        thermometer.position.set( position.plus( THERMOMETER_OFFSET ) );
      }
    } ) );
  }

  return inherit( EnergyUser, BeakerHeater, {
//-------------------------------------------------------------------------
// Methods
//-------------------------------------------------------------------------
      stepInTime: function( dt, incomingEnergy ) {
        if ( isActive() ) {
          // Handle any incoming energy chunks.
          if ( !incomingEnergyChunks.isEmpty() ) {
            for ( var incomingEnergyChunk in incomingEnergyChunks ) {
              if ( incomingEnergyChunk.energyType.get() == EnergyType.ELECTRICAL ) {
                // Add the energy chunk to the list of those under management.
                energyChunkList.add( incomingEnergyChunk );
                // the wire to the heating element.
                electricalEnergyChunkMovers.add( new EnergyChunkPathMover( incomingEnergyChunk, createElectricalEnergyChunkPath( getPosition() ), EFACConstants.ENERGY_CHUNK_VELOCITY ) );
              }
              else {
                // By design, this shouldn't happen, so warn if it does.
                console.log( getClass().getName() + " - Warning: Ignoring energy chunk with unexpected type, type = " + incomingEnergyChunk.energyType.get().toString() );
              }
            }
            incomingEnergyChunks.clear();
          }
          moveElectricalEnergyChunks( dt );
          moveThermalEnergyChunks( dt );
          // Set the proportion of max heat being generated by the heater element.
          if ( (energyChunksVisible.get() && heatingElementEnergyChunkMovers.size() > 0) || (!energyChunksVisible.get() && incomingEnergy.type == EnergyType.ELECTRICAL) ) {
            heatProportion.set( Math.min( incomingEnergy.amount / (EFACConstants.MAX_ENERGY_PRODUCTION_RATE * dt), heatProportion.get() + HEAT_ENERGY_CHANGE_RATE * dt ) );
          }
          else {
            heatProportion.set( Math.max( 0, heatProportion.get() - HEAT_ENERGY_CHANGE_RATE * dt ) );
          }
          // Add energy to the beaker based on heat coming from heat element.
          beaker.changeEnergy( heatProportion.get() * MAX_HEAT_GENERATION_RATE * dt );
          // surrounding air.
          var temperatureGradient = beaker.getTemperature() - ROOM_TEMPERATURE;
          if ( Math.abs( temperatureGradient ) > EFACConstants.TEMPERATURES_EQUAL_THRESHOLD ) {
            var thermalContactArea = (beaker.getRawOutlineRect().getWidth() * 2) + (beaker.getRawOutlineRect().getHeight() * 2) * beaker.fluidLevel.get();
            var thermalEnergyLost = temperatureGradient * HeatTransferConstants.WATER_AIR_HEAT_TRANSFER_FACTOR.get() * thermalContactArea * dt;
            beaker.changeEnergy( -thermalEnergyLost );
            if ( beaker.getEnergyBeyondMaxTemperature() > 0 ) {
              // Prevent the water from going beyond the boiling point.
              beaker.changeEnergy( -beaker.getEnergyBeyondMaxTemperature() );
            }
          }
          if ( beaker.getEnergyChunkBalance() > 0 ) {
            // away, a.k.a. make it "radiate".
            var extractionPoint = new Vector2( beaker.getRect().getMinX() + RAND.nextDouble() * beaker.getRect().getWidth(), beaker.getRect().getMinY() + RAND.nextDouble() * (beaker.getRect().getHeight() * beaker.fluidLevel.get()) );
            var ec = beaker.extractClosestEnergyChunk( extractionPoint );
            if ( ec != null ) {
              // Move to front of z order.
              ec.zPosition.set( 0.0 );
              radiatedEnergyChunkList.add( ec );
              radiatedEnergyChunkMovers.add( new EnergyChunkPathMover( ec, createRadiatedEnergyChunkPath( ec.position.get() ), EFACConstants.ENERGY_CHUNK_VELOCITY ) );
            }
          }
          moveRadiatedEnergyChunks( dt );
        }
      },

      //private
      moveRadiatedEnergyChunks: function( dt ) {
        for ( var radiatedEnergyChunkMover in new ArrayList( radiatedEnergyChunkMovers ) ) {
          radiatedEnergyChunkMover.moveAlongPath( dt );
          if ( radiatedEnergyChunkMover.isPathFullyTraversed() ) {
            // Remove this energy chunk entirely.
            radiatedEnergyChunkList.remove( radiatedEnergyChunkMover.energyChunk );
            radiatedEnergyChunkMovers.remove( radiatedEnergyChunkMover );
          }
        }
      },

      //private
      moveThermalEnergyChunks: function( dt ) {
        for ( var thermalEnergyChunkMover in new ArrayList( heatingElementEnergyChunkMovers ) ) {
          thermalEnergyChunkMover.moveAlongPath( dt );
          if ( thermalEnergyChunkMover.isPathFullyTraversed() ) {
            // chunk.
            beaker.addEnergyChunk( thermalEnergyChunkMover.energyChunk );
            energyChunkList.remove( thermalEnergyChunkMover.energyChunk );
            heatingElementEnergyChunkMovers.remove( thermalEnergyChunkMover );
          }
        }
      },

      //private
      moveElectricalEnergyChunks: function( dt ) {
        for ( var energyChunkMover in new ArrayList( electricalEnergyChunkMovers ) ) {
          energyChunkMover.moveAlongPath( dt );
          if ( energyChunkMover.isPathFullyTraversed() ) {
            // it needs to change into thermal energy.
            electricalEnergyChunkMovers.remove( energyChunkMover );
            energyChunkMover.energyChunk.energyType.set( EnergyType.THERMAL );
            // before moving into the beaker.
            heatingElementEnergyChunkMovers.add( new EnergyChunkPathMover( energyChunkMover.energyChunk, createHeaterElementEnergyChunkPath( energyChunkMover.energyChunk.position.get() ), HEATING_ELEMENT_ENERGY_CHUNK_VELOCITY ) );
          }
        }
      },
      preLoadEnergyChunks: function( incomingEnergyRate ) {
        clearEnergyChunks();
        if ( incomingEnergyRate.amount == 0 || incomingEnergyRate.type != EnergyType.ELECTRICAL ) {
          // No energy chunk pre-loading needed.
          return;
        }
        var dt = 1 / EFACConstants.FRAMES_PER_SECOND;
        var energySinceLastChunk = EFACConstants.ENERGY_PER_CHUNK * 0.99;
        // Simulate energy chunks moving through the system.
        var preLoadComplete = false;
        while ( !preLoadComplete ) {
          energySinceLastChunk += incomingEnergyRate.amount * dt;
          // Determine if time to add a new chunk.
          if ( energySinceLastChunk >= EFACConstants.ENERGY_PER_CHUNK ) {
            var newEnergyChunk = new EnergyChunk( EnergyType.ELECTRICAL, getPosition().plus( OFFSET_TO_LEFT_SIDE_OF_WIRE ), energyChunksVisible );
            energyChunkList.add( newEnergyChunk );
            // the wire to the heating element.
            electricalEnergyChunkMovers.add( new EnergyChunkPathMover( newEnergyChunk, createElectricalEnergyChunkPath( getPosition() ), EFACConstants.ENERGY_CHUNK_VELOCITY ) );
            // Update energy since last chunk.
            energySinceLastChunk = energySinceLastChunk - EFACConstants.ENERGY_PER_CHUNK;
          }
          moveElectricalEnergyChunks( dt );
          if ( heatingElementEnergyChunkMovers.size() > 0 ) {
            // An energy chunks has made it to the heating element, which completes the preload.
            preLoadComplete = true;
          }
        }
      },
      deactivate: function() {
        super.deactivate();
        heatProportion.set( 0.0 );
        beaker.reset();
        beaker.position.set( getPosition().plus( BEAKER_OFFSET ) );
      },
      clearEnergyChunks: function() {
        super.clearEnergyChunks();
        electricalEnergyChunkMovers.clear();
        heatingElementEnergyChunkMovers.clear();
        radiatedEnergyChunkMovers.clear();
        radiatedEnergyChunkList.clear();
      },
      getUserComponent: function() {
        return EnergyFormsAndChangesSimSharing.UserComponents.selectBeakerHeaterButton;
      },

      //private
      createHeaterElementEnergyChunkPath: function( startPosition ) {
        return [].withAnonymousClassBody( {
          initializer: function() {
            // coordinated with the burner element image.
            var angle = RAND.nextBoolean() ? RAND.nextDouble() * Math.PI * 0.45 : -RAND.nextDouble() * Math.PI * 0.3;
            add( startPosition.plus( new Vector2( 0, HEATER_ELEMENT_2D_HEIGHT ).getRotatedInstance( angle ) ) );
          }
        } );
      },

      //private
      createElectricalEnergyChunkPath: function( centerPosition ) {
        return [].withAnonymousClassBody( {
          initializer: function() {
            add( centerPosition.plus( OFFSET_TO_LEFT_SIDE_OF_WIRE_BEND ) );
            add( centerPosition.plus( OFFSET_TO_FIRST_WIRE_CURVE_POINT ) );
            add( centerPosition.plus( OFFSET_TO_SECOND_WIRE_CURVE_POINT ) );
            add( centerPosition.plus( OFFSET_TO_THIRD_WIRE_CURVE_POINT ) );
            add( centerPosition.plus( OFFSET_TO_BOTTOM_OF_CONNECTOR ) );
            add( centerPosition.plus( OFFSET_TO_CONVERSION_POINT ) );
          }
        } );
      },

      //private
      createRadiatedEnergyChunkPath: function( startPosition ) {
        // Empirically chosen.
        var numDirectionChanges = 4;
        var nominalTravelVector = new Vector2( 0, RADIATED_ENERGY_CHUNK_TRAVEL_DISTANCE / numDirectionChanges );
        var pathPoints = [];
        // good, making the chunk move straight up out of the beaker.
        var currentPosition = startPosition.plus( nominalTravelVector );
        pathPoints.add( currentPosition );
        // Add the remaining points in the path.
        for ( var i = 0; i < numDirectionChanges - 1; i++ ) {
          var movement = nominalTravelVector.getRotatedInstance( (RAND.nextDouble() - 0.5) * Math.PI / 4 );
          currentPosition = currentPosition.plus( movement );
          pathPoints.add( currentPosition );
        }
        return pathPoints;
      }
    },
//statics
    {
      WIRE_STRAIGHT_IMAGE: WIRE_STRAIGHT_IMAGE,
      WIRE_CURVE_IMAGE: WIRE_CURVE_IMAGE,
      ELEMENT_BASE_BACK_IMAGE: ELEMENT_BASE_BACK_IMAGE,
      ELEMENT_BASE_FRONT_IMAGE: ELEMENT_BASE_FRONT_IMAGE,
      HEATER_ELEMENT_OFF_IMAGE: HEATER_ELEMENT_OFF_IMAGE,
      HEATER_ELEMENT_ON_IMAGE: HEATER_ELEMENT_ON_IMAGE
    } );
} );

