// Copyright 2002-2012, University of Colorado
/**
 * Class that is used to move an energy chunk along a path.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var List = require( 'java.util.List' );
  var Vector2 = require( 'DOT/Vector2' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );

  function EnergyChunkPathMover( energyChunk, path, velocity ) {
    this.energyChunk;

    //private
    this.path;

    //private
    this.velocity;

    //private
    this.nextPoint;

    //private
    this.pathFullyTraversed = false;
    this.energyChunk = energyChunk;
    this.path = path;
    this.velocity = velocity;
    if ( path.isEmpty() ) {
      throw new IllegalArgumentException( "Path must have at least one point." );
    }
    nextPoint = path.get( 0 );
  }

  return inherit( Object, EnergyChunkPathMover, {
    moveAlongPath: function( time ) {
      var distanceToTravel = time * velocity;
      while ( distanceToTravel > 0 && !pathFullyTraversed ) {
        if ( distanceToTravel < energyChunk.position.get().distance( nextPoint ) ) {
          // Not arriving at destination next point yet, so just move towards it.
          var velocityVector = new Vector2( distanceToTravel, 0 ).getRotatedInstance( nextPoint.minus( energyChunk.position.get() ).getAngle() );
          energyChunk.position.set( energyChunk.position.get().plus( velocityVector ) );
          // No remaining distance.
          distanceToTravel = 0;
        }
        else {
          // Arrived at next destination point.
          distanceToTravel = distanceToTravel - energyChunk.position.get().distance( nextPoint );
          energyChunk.position.set( nextPoint );
          if ( nextPoint == path.get( path.size() - 1 ) ) {
            // At the end.
            pathFullyTraversed = true;
          }
          else {
            // Set the next destination point.
            nextPoint = path.get( path.indexOf( nextPoint ) + 1 );
          }
        }
      }
    },
    getFinalDestination: function() {
      return path.get( path.size() - 1 );
    },
    isPathFullyTraversed: function() {
      return pathFullyTraversed;
    }
  } );
} );

