// Copyright 2002-2012, University of Colorado
/**
 * Base class for energy converters, i.e. model elements that take energy from
 * a source and convert it to something else (such as mechanical to electrical)
 * and then supply it to an energy user.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Image = require( 'java.awt.Image' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );

  function EnergyConverter( iconImage ) {
    this.incomingEnergyChunks = [];
    this.outgoingEnergyChunks = [];
    EnergySystemElement.call( this, iconImage );
  }

  return inherit( EnergySystemElement, EnergyConverter, {
    stepInTime: function( dt, incomingEnergy ) {},
    /**
     * Set up the energy chunks contained by this energy source as though they
     * have been full propagated through the system.
     *
     * @param incomingEnergyRate Incoming energy in joules/sec.
     */
    preLoadEnergyChunks: function( incomingEnergyRate ) {},
    /**
     * Get the energy chunks that this source wants to transfer to the next
     * energy system element.  Reading clears the list.
     *
     * @return list of energy chunks to transfer.
     */
    extractOutgoingEnergyChunks: function() {
      var retVal = new ArrayList( outgoingEnergyChunks );
      energyChunkList.removeAll( outgoingEnergyChunks );
      outgoingEnergyChunks.clear();
      return retVal;
    },
    /**
     * Inject a list of energy chunks into this energy system element.  Once
     * injected, it is the system's responsibility to move, convert, and
     * otherwise manage them.
     *
     * @param energyChunks List of energy chunks to inject.
     */
    injectEnergyChunks: function( energyChunks ) {
      incomingEnergyChunks.addAll( energyChunks );
    },
    /**
     * Get the amount and type of energy currently being produced.
     *
     * @return amount and type of energy.
     */
    getEnergyOutputRate: function() {},
    clearEnergyChunks: function() {
      super.clearEnergyChunks();
      incomingEnergyChunks.clear();
      outgoingEnergyChunks.clear();
    }
  } );
} );

