// Copyright 2002-2012, University of Colorado
/**
 * Base class for energy sources, converters, and users.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Image = require( 'java.awt.Image' );
  var Vector2 = require( 'DOT/Vector2' );
  var Property = require( 'AXON/Property' );
  var ChangeObserver = require( 'edu.colorado.phet.common.phetcommon.model.property.ChangeObserver' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var ObservableArray = require( 'AXON/ObservableArray' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );

  function EnergySystemElement( iconImage ) {
    // List of energy chunks contained and managed by this energy system element.
    this.energyChunkList = new ObservableArray();
    // Image that is used as an icon in the view to represent this element.

    //private
    this.iconImage;
    // State variable that tracks whether this element is currently active,
    // meaning that it is generating, transforming, and/or consuming energy.

    //private
    this.active = new BooleanProperty( false );
    this.iconImage = iconImage;
    // Move all energy chunks with this element as it moves.
    getObservablePosition().addObserver( new ChangeObserver().withAnonymousClassBody( {
      update: function( newPosition, oldPosition ) {
        var deltaPosition = newPosition.minus( oldPosition );
        for ( var energyChunk in energyChunkList ) {
          energyChunk.translate( deltaPosition );
        }
      }
    } ) );
  }

  return inherit( PositionableFadableModelElement, EnergySystemElement, {
    getIconImage: function() {
      return iconImage;
    },
    getUserComponent: function() {},
    activate: function() {
      active.set( true );
    },
    deactivate: function() {
      active.set( false );
      // All energy chunks go away when an energy system element is deactivated.
      clearEnergyChunks();
    },
    getObservableActiveState: function() {
      return active;
    },
    isActive: function() {
      return active.get();
    },
    /**
     * Remove all energy chunks being managed by this energy system element.
     */
    clearEnergyChunks: function() {
      energyChunkList.clear();
    }
  } );
} );

