// Copyright 2002-2012, University of Colorado
/**
 * Base class for energy users, i.e. model elements that take energy from
 * an energy converter and do something with it, such as producing light or
 * heat.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );

  function EnergyUser( iconImage ) {
    this.incomingEnergyChunks = [];
    EnergySystemElement.call( this, iconImage );
  }

  return inherit( EnergySystemElement, EnergyUser, {
    stepInTime: function( dt, incomingEnergy ) {},
    /**
     * Set up the energy chunks contained by this energy source as though they
     * have been full propagated through the system.
     *
     * @param incomingEnergyRate Incoming energy in joules/sec.
     */
    preLoadEnergyChunks: function( incomingEnergyRate ) {},
    /**
     * Inject a list of energy chunks into this energy system element.  Once
     * injected, it is the system's responsibility to move, convert, and
     * otherwise manage them.
     *
     * @param energyChunks List of energy chunks to inject.
     */
    injectEnergyChunks: function( energyChunks ) {
      incomingEnergyChunks.addAll( energyChunks );
    },
    clearEnergyChunks: function() {
      super.clearEnergyChunks();
      incomingEnergyChunks.clear();
    }
  } );
} );

