// Copyright 2002-2012, University of Colorado
/**
 * Class that provides information about an image that is used in the view
 * representation of a model element.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BufferedImage = require( 'java.awt.image.BufferedImage' );
  var Vector2 = require( 'DOT/Vector2' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );

// Scaling factor that uses the image width to define its size in the model.

  //private
  var DEFAULT_SCALING_FACTOR = 1 / EFACConstants.ENERGY_SYSTEMS_MVT_SCALE_FACTOR;

  function ModelElementImage( image, centerToCenterOffset ) {

    //private
    this.image;
    // Width of the image in model units (meters).  Height is derived from aspect ratio of image.

    //private
    this.width;
    // Offset in model units (meters) from the center of the position of the
    // model element that owns this image to the center of the image.

    //private
    this.centerToCenterOffset;
    this( image, image.getWidth() * DEFAULT_SCALING_FACTOR, centerToCenterOffset );
  }

  function ModelElementImage( image, width, centerToCenterOffset ) {

    //private
    this.image;
    // Width of the image in model units (meters).  Height is derived from aspect ratio of image.

    //private
    this.width;
    // Offset in model units (meters) from the center of the position of the
    // model element that owns this image to the center of the image.

    //private
    this.centerToCenterOffset;
    this.image = image;
    this.width = width;
    this.centerToCenterOffset = centerToCenterOffset;
  }

  return inherit( Object, ModelElementImage, {
    getImage: function() {
      return image;
    },
    getWidth: function() {
      return width;
    },
    getCenterToCenterOffset: function() {
      return centerToCenterOffset;
    },
    getHeight: function() {
      // Height is based on specified width and the aspect ratio of the image.
      return (width / image.getWidth()) * image.getHeight();
    }
  } );
} );

