// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents the beaker heater view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Vector2 = require( 'DOT/Vector2' );
  var IClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.IClock' );
  var Property = require( 'AXON/Property' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var BeakerView = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/BeakerView' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var BeakerHeater = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/BeakerHeater' );
  var SensingThermometerNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/view/SensingThermometerNode' );
  var ThermometerNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/view/ThermometerNode' );
  var PImage = require( 'edu.umd.cs.piccolo.nodes.PImage' );

  function BeakerHeaterNode( clock, beakerHeater, energyChunksVisible, mvt ) {
    ImageBasedEnergySystemElementNode.call( this, beakerHeater, mvt );
    // layer that will contain the energy chunks.
    addImageNode( BeakerHeater.WIRE_STRAIGHT_IMAGE );
    addImageNode( BeakerHeater.WIRE_CURVE_IMAGE );
    addImageNode( BeakerHeater.ELEMENT_BASE_BACK_IMAGE );
    addImageNode( BeakerHeater.HEATER_ELEMENT_OFF_IMAGE );
    var energizedCoil = addImageNode( BeakerHeater.HEATER_ELEMENT_ON_IMAGE );
    addChild( new EnergyChunkLayer( beakerHeater.energyChunkList, beakerHeater.getObservablePosition(), mvt ) );
    addImageNode( BeakerHeater.ELEMENT_BASE_FRONT_IMAGE );
    // own offset in model space.
    var compensatingMvt = ModelViewTransform.createOffsetScaleMapping( new Vector2( -mvt.modelToViewDeltaX( beakerHeater.getPosition().x ), -mvt.modelToViewDeltaY( beakerHeater.getPosition().y ) ), mvt.getTransform().getScaleX(), mvt.getTransform().getScaleY() );
    var beakerView = new BeakerView( clock, beakerHeater.beaker, energyChunksVisible, compensatingMvt );
    addChild( beakerView.getBackNode() );
    addChild( new EnergyChunkLayer( beakerHeater.radiatedEnergyChunkList, beakerHeater.getObservablePosition(), mvt ) );
    addChild( beakerView.getFrontNode() );
    // Add the thermometer.
    addChild( new SensingThermometerNode( beakerHeater.thermometer ).withAnonymousClassBody( {
      initializer: function() {
        setPickable( false );
        setChildrenPickable( false );
        setOffset( 75, -220 );
      }
    } ) );
    // appear to heat up.
    beakerHeater.heatProportion.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( litProportion ) {
        energizedCoil.setTransparency( litProportion.floatValue() );
      }
    } ) );
  }

  return inherit( ImageBasedEnergySystemElementNode, BeakerHeaterNode, {
  } );
} );

