// Copyright 2002-2012, University of Colorado
/**
 * PNode that represents a belt that connects two circular items, like a fan
 * belt in an automobile.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var Shape = require( 'java.awt.Shape' );
  var Stroke = require( 'java.awt.Stroke' );
  var Area = require( 'java.awt.geom.Area' );
  var Ellipse2D = require( 'java.awt.geom.Ellipse2D' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var Vector2 = require( 'DOT/Vector2' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var ShapeUtils = require( 'edu.colorado.phet.common.phetcommon.view.util.ShapeUtils' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var Belt = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/Belt' );
  var Node = require( 'SCENERY/nodes/Node' );


  //private
  var BELT_STROKE = new BasicStroke( 5, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND );

  //private
  var BELT_COLOR = Color.BLACK;

  function BeltNode( belt, mvt ) {
    // Create the wheel shapes.
    var wheel1Radius = mvt.modelToViewDeltaX( belt.wheel1Radius );
    var wheel1Center = mvt.modelToView( belt.wheel1Center );
    var wheel1Shape = new Ellipse2D.Number( wheel1Center.getX() - wheel1Radius, wheel1Center.getY() - wheel1Radius, wheel1Radius * 2, wheel1Radius * 2 );
    var wheel2Radius = mvt.modelToViewDeltaX( belt.wheel2Radius );
    var wheel2Center = mvt.modelToView( belt.wheel2Center );
    var wheel2Shape = new Ellipse2D.Number( wheel2Center.getX() - wheel2Radius, wheel2Center.getY() - wheel2Radius, wheel2Radius * 2, wheel2Radius * 2 );
    // Create a shape that will connect the two circles in a belt-like shape.
    var points = [].withAnonymousClassBody( {
      initializer: function() {
        var wheel1CenterToWheelTwoCenter = wheel2Center.minus( wheel1Center );
        var wheel1ToOneEdge = wheel1CenterToWheelTwoCenter.getPerpendicularVector().getInstanceOfMagnitude( wheel1Radius );
        add( wheel1Center.plus( wheel1ToOneEdge ) );
        add( wheel1Center.minus( wheel1ToOneEdge ) );
        var wheel2ToOneEdge = wheel1CenterToWheelTwoCenter.getPerpendicularVector().getInstanceOfMagnitude( wheel2Radius );
        add( wheel2Center.minus( wheel2ToOneEdge ) );
        add( wheel2Center.plus( wheel2ToOneEdge ) );
      }
    } );
    // Combine the shapes using constructive area geometry.
    var overallShape = new Area( ShapeUtils.createShapeFromPoints( points ) );
    overallShape.add( new Area( wheel1Shape ) );
    overallShape.add( new Area( wheel2Shape ) );
    // Add the node.
    addChild( new PhetPPath( overallShape, BELT_STROKE, BELT_COLOR ) );
    // Control the visibility.
    belt.isVisible.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( isVisible ) {
        setVisible( isVisible );
      }
    } ) );
  }

  return inherit( Node, BeltNode, {
  } );
} );

