// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents the biker in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var ActionEvent = require( 'java.awt.event.ActionEvent' );
  var ActionListener = require( 'java.awt.event.ActionListener' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var SettableProperty = require( 'edu.colorado.phet.common.phetcommon.model.property.SettableProperty' );
  var SimpleObserver = require( 'edu.colorado.phet.common.phetcommon.util.SimpleObserver' );
  var Function0 = require( 'edu.colorado.phet.common.phetcommon.util.function.Function0' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetFont = require( 'edu.colorado.phet.common.phetcommon.view.util.PhetFont' );
  var ButtonNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ButtonNode' );
  var ControlPanelNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ControlPanelNode' );
  var TextButtonNode = require( 'edu.colorado.phet.common.piccolophet.nodes.TextButtonNode' );
  var HSliderNode = require( 'edu.colorado.phet.common.piccolophet.nodes.slider.HSliderNode' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var Biker = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/Biker' );
  var Node = require( 'SCENERY/nodes/Node' );


  // static class: CrankRateSlider
  var CrankRateSlider =

    //private
    define( function( require ) {
      function CrankRateSlider( value ) {
        addChild( new ControlPanelNode( new HSliderNode( EnergyFormsAndChangesSimSharing.UserComponents.bicyclePedalRateSlider, 0, Biker.MAX_ANGULAR_VELOCITY_OF_CRANK, value ), EFACConstants.CONTROL_PANEL_BACKGROUND_COLOR ) );
      }

      return inherit( Node, CrankRateSlider, {
      } );
    } );
  ;
  function BikerNode( biker, mvt ) {

    //private
    this.feedMeButton;
    PositionableFadableModelElementNode.call( this, biker, mvt );
    // Create and add the various image nodes.
    var spokesImage = new ModelElementImageNode( Biker.REAR_WHEEL_SPOKES_IMAGE, mvt );
    addChild( spokesImage );
    var backLegRootNode = new Node();
    addChild( backLegRootNode );
    addChild( new ModelElementImageNode( Biker.FRAME_IMAGE, mvt ) );
    var frontLegRootNode = new Node();
    addChild( frontLegRootNode );
    // Add slider that will control pedaling rate.
    addChild( new CrankRateSlider( biker.targetCrankAngularVelocity ).withAnonymousClassBody( {
      initializer: function() {
        setOffset( -getFullBoundsReference().width / 2, 100 );
      }
    } ) );
    // Add the images that will animate the front and back legs.
    var backLegImageNodes = [];
    var frontLegImageNodes = [];
    for ( var i = 0; i < Biker.NUM_LEG_IMAGES; i++ ) {
      var backLegImageNode = new ModelElementImageNode( Biker.BACK_LEG_IMAGES.get( i ), mvt );
      backLegImageNode.setVisible( false );
      backLegImageNodes.add( backLegImageNode );
      backLegRootNode.addChild( backLegImageNode );
      var frontLegImageNode = new ModelElementImageNode( Biker.FRONT_LEG_IMAGES.get( i ), mvt );
      frontLegImageNode.setVisible( false );
      frontLegImageNodes.add( frontLegImageNode );
      frontLegRootNode.addChild( frontLegImageNode );
    }
    biker.getCrankAngle().addObserver( new VoidFunction1().withAnonymousClassBody( {
      var visibleFrontLeg = frontLegImageNodes.get( 0 ),
      var visibleBackLeg = backLegImageNodes.get( 0 ),
      apply: function( rotationalAngle ) {
        var currentImageIndex = Biker.mapAngleToImageIndex( rotationalAngle );
        visibleFrontLeg.setVisible( false );
        visibleBackLeg.setVisible( false );
        visibleFrontLeg = frontLegImageNodes.get( currentImageIndex );
        visibleFrontLeg.setVisible( true );
        visibleBackLeg = backLegImageNodes.get( currentImageIndex );
        visibleBackLeg.setVisible( true );
      }
    } ) );
    // Add the upper body.
    var upperBodyNormal = new ModelElementImageNode( Biker.RIDER_NORMAL_UPPER_BODY_IMAGE, mvt );
    addChild( upperBodyNormal );
    var upperBodyTired = new ModelElementImageNode( Biker.RIDER_TIRED_UPPER_BODY_IMAGE, mvt );
    addChild( upperBodyTired );
    // Add the energy chunk layers.
    addChild( new EnergyChunkLayer( biker.energyChunkList, biker.getObservablePosition(), mvt ) );
    // Add button to replenish the biker's energy.
    feedMeButton = new TextButtonNode( EnergyFormsAndChangesResources.Strings.FEED_ME, new PhetFont( 18 ), new Color( 0, 220, 0 ) );
    feedMeButton.setOffset( -feedMeButton.getFullBoundsReference().width / 2, upperBodyNormal.getFullBoundsReference().getMinY() - 45 );
    feedMeButton.addActionListener( new ActionListener().withAnonymousClassBody( {
      actionPerformed: function( e ) {
        biker.replenishEnergyChunks();
      }
    } ) );
    biker.bikerHasEnergy.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( hasEnergy ) {
        feedMeButton.setVisible( !hasEnergy );
        upperBodyNormal.setVisible( hasEnergy );
        upperBodyTired.setVisible( !hasEnergy );
      }
    } ) );
    addChild( feedMeButton );
    // Add and observer that will turn the back wheel.
    var wheelRotationPoint = new Vector2( spokesImage.getFullBoundsReference().getCenterX(), spokesImage.getFullBoundsReference().getCenterY() );
    biker.getRearWheelAngle().addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( angle ) {
        // Limit this to one rotation.
        assert && assert( angle < 2 * Math.PI );
        // invert the angle in the following calculation.
        var compensatedAngle = (2 * Math.PI - spokesImage.getRotation()) % (2 * Math.PI);
        var delta = angle - compensatedAngle;
        spokesImage.rotateAboutPoint( -delta, wheelRotationPoint );
      }
    } ) );
  }

  return inherit( PositionableFadableModelElementNode, BikerNode, {
  } );
} );

