// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents the water-powered generator in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var ElectricalGenerator = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/ElectricalGenerator' );
  var Node = require( 'SCENERY/nodes/Node' );

  function ElectricalGeneratorNode( generator, mvt ) {
    PositionableFadableModelElementNode.call( this, generator, mvt );
    // Create and add the various image nodes and energy chunk layers.
    addChild( new ModelElementImageNode( ElectricalGenerator.WIRE_CURVED_IMAGE, mvt ) );
    addChild( new EnergyChunkLayer( generator.electricalEnergyChunks, generator.getObservablePosition(), mvt ) );
    addChild( new ModelElementImageNode( ElectricalGenerator.HOUSING_IMAGE, mvt ) );
    addChild( new ModelElementImageNode( ElectricalGenerator.CONNECTOR_IMAGE, mvt ) );
    var spokesNode = new ModelElementImageNode( ElectricalGenerator.SHORT_SPOKES_IMAGE, mvt );
    addChild( spokesNode );
    var paddlesNode = new ModelElementImageNode( ElectricalGenerator.WHEEL_PADDLES_IMAGE, mvt );
    addChild( paddlesNode );
    addChild( new ModelElementImageNode( ElectricalGenerator.WHEEL_HUB_IMAGE, mvt ) );
    addChild( new EnergyChunkLayer( generator.hiddenEnergyChunks, generator.getObservablePosition(), mvt ) );
    addChild( new EnergyChunkLayer( generator.energyChunkList, generator.getObservablePosition(), mvt ) );
    // Update the rotation of the wheel image based on model value.
    var wheelRotationPoint = new Vector2( paddlesNode.getFullBoundsReference().getCenterX(), paddlesNode.getFullBoundsReference().getCenterY() );
    generator.getWheelRotationalAngle().addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( angle ) {
        var delta = -angle - paddlesNode.getRotation();
        paddlesNode.rotateAboutPoint( delta, wheelRotationPoint );
        spokesNode.rotateAboutPoint( delta, wheelRotationPoint );
      }
    } ) );
    // Hide the paddles and show the spokes when in direct coupling mode.
    generator.directCouplingMode.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( directCouplingMode ) {
        paddlesNode.setVisible( !directCouplingMode );
        spokesNode.setVisible( directCouplingMode );
      }
    } ) );
  }

  return inherit( PositionableFadableModelElementNode, ElectricalGeneratorNode, {
  } );
} );

