// Copyright 2002-2012, University of Colorado
/**
 * Legend for the various different energy chunks, i.e. thermal, electrical,
 * etc.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Font = require( 'SCENERY/util/Font' );
  var GridBagConstraints = require( 'java.awt.GridBagConstraints' );
  var GridBagLayout = require( 'java.awt.GridBagLayout' );
  var Property = require( 'AXON/Property' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetFont = require( 'edu.colorado.phet.common.phetcommon.view.util.PhetFont' );
  var ControlPanelNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ControlPanelNode' );
  var PhetPText = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPText' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );
  var EnergyChunkNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkNode' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PImage = require( 'edu.umd.cs.piccolo.nodes.PImage' );
  var SwingLayoutNode = require( 'edu.umd.cs.piccolox.swing.SwingLayoutNode' );


  //private
  var LEGEND_ENTRY_FONT = new PhetFont( 14 );

  function EnergyChunkLegend() {
    var contents = new SwingLayoutNode( new GridBagLayout() ).withAnonymousClassBody( {
      initializer: function() {
        var constraints = new GridBagConstraints();
        // Add the title.
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 2;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.FORMS_OF_ENERGY, new PhetFont( 14, true ) ), constraints );
        // Add the legend entries.
        constraints.gridy++;
        constraints.gridwidth = 1;
        constraints.gridx = 0;
        constraints.anchor = GridBagConstraints.LINE_START;
        constraints.insets.top = 10;
        constraints.insets.left = 0;
        addChild( EnergyChunkNode.createEnergyChunkNode( EnergyType.MECHANICAL ), constraints );
        constraints.gridx++;
        constraints.insets.left = 5;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.MECHANICAL, LEGEND_ENTRY_FONT ), constraints );
        constraints.gridy++;
        constraints.gridx = 0;
        constraints.insets.top = 5;
        constraints.insets.left = 0;
        addChild( EnergyChunkNode.createEnergyChunkNode( EnergyType.ELECTRICAL ), constraints );
        constraints.gridx++;
        constraints.insets.left = 5;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.ELECTRICAL, LEGEND_ENTRY_FONT ), constraints );
        constraints.gridy++;
        constraints.gridx = 0;
        constraints.insets.left = 0;
        addChild( EnergyChunkNode.createEnergyChunkNode( EnergyType.THERMAL ), constraints );
        constraints.gridx++;
        constraints.insets.left = 5;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.THERMAL, LEGEND_ENTRY_FONT ), constraints );
        constraints.gridy++;
        constraints.gridx = 0;
        constraints.insets.left = 0;
        addChild( EnergyChunkNode.createEnergyChunkNode( EnergyType.LIGHT ), constraints );
        constraints.gridx++;
        constraints.insets.left = 5;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.LIGHT, LEGEND_ENTRY_FONT ), constraints );
        constraints.gridy++;
        constraints.gridx = 0;
        constraints.insets.left = 0;
        addChild( EnergyChunkNode.createEnergyChunkNode( EnergyType.CHEMICAL ), constraints );
        constraints.gridx++;
        constraints.insets.left = 5;
        addChild( new PhetPText( EnergyFormsAndChangesResources.Strings.CHEMICAL, LEGEND_ENTRY_FONT ), constraints );
      }
    } );
    // Create the control panel.
    addChild( new ControlPanelNode( contents, new Color( 250, 250, 250 ) ) );
  }

  return inherit( Node, EnergyChunkLegend, {
  } );
} );

