// Copyright 2002-2012, University of Colorado
/**
 * PNode that allows the user to select the various elements contained within
 * a carousel by presenting a set of radio-style push buttons, each with an
 * icon image of the selection that it represents.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var Image = require( 'java.awt.Image' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var RadioButtonStripControlPanelNode = require( 'edu.colorado.phet.common.piccolophet.nodes.radiobuttonstrip.RadioButtonStripControlPanelNode' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var EnergySystemElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/EnergySystemElement' );
  var EnergySystemElementCarousel = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/EnergySystemElementCarousel' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PImage = require( 'edu.umd.cs.piccolo.nodes.PImage' );

// In screen coordinates, which is close to pixels.

  //private
  var BUTTON_IMAGE_HEIGHT_OR_WIDTH = 50;

  function EnergySystemElementSelector( carousel ) {
    var buttonElementList = [].withAnonymousClassBody( {
      initializer: function() {
        for ( var i = 0; i < carousel.getNumElements(); i++ ) {
          var buttonImage = carousel.getElement( i ).getIconImage();
          var buttonImageNode = new PImage( buttonImage ).withAnonymousClassBody( {
            initializer: function() {
              if ( getFullBoundsReference().width > getFullBoundsReference().height ) {
                setScale( BUTTON_IMAGE_HEIGHT_OR_WIDTH / getFullBoundsReference().width );
              }
              else {
                setScale( BUTTON_IMAGE_HEIGHT_OR_WIDTH / getFullBoundsReference().height );
              }
            }
          } );
          add( new RadioButtonStripControlPanelNode.Element( buttonImageNode, i, carousel.getElement( i ).getUserComponent() ) );
        }
      }
    } );
    addChild( new RadioButtonStripControlPanelNode( carousel.targetIndex, buttonElementList, 5, EFACConstants.CONTROL_PANEL_BACKGROUND_COLOR, EFACConstants.CONTROL_PANEL_OUTLINE_STROKE, EFACConstants.CONTROL_PANEL_OUTLINE_COLOR, 4 ) );
  }

  return inherit( Node, EnergySystemElementSelector, {
  } );
} );

