// Copyright 2002-2012, University of Colorado
/**
 * Class the depicts a line that starts at a point in space and then fades
 * out.  This was originally created to represent rays of light emanating from
 * a light source, but it may have other uses.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var GradientPaint = require( 'java.awt.GradientPaint' );
  var Paint = require( 'java.awt.Paint' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Line2D = require( 'java.awt.geom.Line2D' );
  var JFrame = require( 'javax.swing.JFrame' );
  var Vector2 = require( 'DOT/Vector2' );
  var PhetPCanvas = require( 'edu.colorado.phet.common.piccolophet.PhetPCanvas' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PDimension = require( 'edu.umd.cs.piccolo.util.PDimension' );

  function FadingLineNode( origin, endpoint, startColor, fadeCoefficient, lineThickness ) {

    //private
    this.intensityAtEndPoint;
    // Can't have negative fade.
    assert && assert( fadeCoefficient >= 0 );
    intensityAtEndPoint = Math.round( startColor.getAlpha() * Math.pow( Math.E, -fadeCoefficient * origin.distance( endpoint ) ) );
    var gradientPaint;
    if ( intensityAtEndPoint == 0 ) {
      var zeroIntensityDistance = (Math.log( startColor.getAlpha() ) - Math.log( 0.4999 )) / fadeCoefficient;
      var zeroIntensityPoint = origin.plus( new Vector2( zeroIntensityDistance, 0 ).getRotatedInstance( endpoint.minus( origin ).getAngle() ) );
      gradientPaint = new GradientPaint( origin.x, origin.y, startColor, zeroIntensityPoint.x, zeroIntensityPoint.y, new Color( startColor.getRed(), startColor.getGreen(), startColor.getBlue(), 0 ) );
    }
    else {
      gradientPaint = new GradientPaint( origin.x, origin.y, startColor, endpoint.x, endpoint.y, new Color( startColor.getRed(), startColor.getGreen(), startColor.getBlue(), intensityAtEndPoint ) );
    }
    addChild( new PhetPPath( new Line2D.Number( origin.toPoint2D(), endpoint.toPoint2D() ), new BasicStroke( lineThickness ), gradientPaint ) );
  }

  return inherit( Node, FadingLineNode, {
    getOpacityAtEndpoint: function() {
      return intensityAtEndPoint;
    },
// Test harness.
    main: function( args ) {
      var stageSize = new PDimension( 400, 600 );
      var canvas = new PhetPCanvas();
      // Set up the canvas-screen transform.
      canvas.setWorldTransformStrategy( new PhetPCanvas.CenteredStage( canvas, stageSize ) );
      var rootNode = new Node();
      canvas.getLayer().addChild( rootNode );
      rootNode.addChild( new FadingLineNode( new Vector2( 10, 10 ), new Vector2( 100, 100 ), Color.ORANGE, 0.05, 2 ) );
      // Boiler plate app stuff.
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.setSize( stageSize.getWidth(), stageSize.getHeight() );
      frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
      // Center.
      frame.setLocationRelativeTo( null );
      frame.setVisible( true );
    }
  } );
} );

