// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents a faucet and flowing water in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var FaucetNode = require( 'edu.colorado.phet.common.piccolophet.nodes.faucet.FaucetNode' );
  var VSliderNode = require( 'edu.colorado.phet.common.piccolophet.nodes.slider.VSliderNode' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var FaucetAndWater = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/FaucetAndWater' );
  var WaterDrop = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/WaterDrop' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PPath = require( 'edu.umd.cs.piccolo.nodes.PPath' );

// In screen coords, which are close to pixels.

  //private
  var FAUCET_NODE_HORIZONTAL_LENGTH = 700;

  function FaucetAndWaterNode( faucet, mvt ) {
    PositionableFadableModelElementNode.call( this, faucet, mvt );
    // Create the faucet.
    var faucetNode = new FaucetNode( EnergyFormsAndChangesSimSharing.UserComponents.faucet, 1, faucet.flowProportion, faucet.getObservableActiveState(), FAUCET_NODE_HORIZONTAL_LENGTH, 40, false );
    // Make it a little smaller than default.  Looks better in this sim.
    faucetNode.setScale( 0.9 );
    faucetNode.setOffset( -faucetNode.getGlobalOutputCenter().getX() + mvt.modelToViewDeltaX( FaucetAndWater.OFFSET_FROM_CENTER_TO_WATER_ORIGIN.getX() ), -faucetNode.getGlobalOutputCenter().getY() + mvt.modelToViewDeltaY( FaucetAndWater.OFFSET_FROM_CENTER_TO_WATER_ORIGIN.getY() ) );
    // Create the water.
    var waterNode = new PhetPPath( EFACConstants.WATER_COLOR_OPAQUE );
    waterNode.setOffset( -mvt.modelToViewX( 0 ) + mvt.modelToViewDeltaX( FaucetAndWater.OFFSET_FROM_CENTER_TO_WATER_ORIGIN.getX() ), -mvt.modelToViewY( 0 ) + mvt.modelToViewDeltaY( FaucetAndWater.OFFSET_FROM_CENTER_TO_WATER_ORIGIN.getY() ) );
    // Create the water, which consists of a set of water drops.
    var waterLayer = new Node();
    faucet.waterDrops.addElementAddedObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( addedWaterDrop ) {
        var waterDropNode = new WaterDropNode( addedWaterDrop, mvt );
        waterLayer.addChild( waterDropNode );
        faucet.waterDrops.addElementRemovedObserver( new VoidFunction1().withAnonymousClassBody( {
          apply: function( removedWaterDrop ) {
            if ( removedWaterDrop == addedWaterDrop ) {
              faucet.waterDrops.removeElementAddedObserver( this );
              waterLayer.removeChild( waterDropNode );
            }
          }
        } ) );
      }
    } ) );
    // Create the energy chunk layer.
    var energyChunkLayer = new EnergyChunkLayer( faucet.energyChunkList, faucet.getObservablePosition(), mvt );
    // Add the nodes in the order that creates the desired layering.
    addChild( waterLayer );
    addChild( energyChunkLayer );
    addChild( faucetNode );
  }

  return inherit( PositionableFadableModelElementNode, FaucetAndWaterNode, {
  } );
} );

