// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents the fluorescent light bulb in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Vector2 = require( 'DOT/Vector2' );
  var Property = require( 'AXON/Property' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var FluorescentLightBulb = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/FluorescentLightBulb' );
  var LightBulb = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/LightBulb' );
  var Node = require( 'SCENERY/nodes/Node' );

  function FluorescentLightBulbNode( lightBulb, energyChunksVisible, mvt ) {
    PositionableFadableModelElementNode.call( this, lightBulb, mvt );
    // Add the light rays.
    var lightRays = new LightRays( new Vector2( 0, 0 ), 30, 400, 20, Math.PI / 4, Color.YELLOW );
    addChild( lightRays );
    energyChunksVisible.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( energyChunksVisible ) {
        // Only show the light rays when the energy chunks are not shown.
        lightRays.setVisible( !energyChunksVisible );
      }
    } ) );
    // Add the images and the layer that will contain the energy chunks.
    addChild( new ModelElementImageNode( LightBulb.WIRE_FLAT_IMAGE, mvt ) );
    addChild( new ModelElementImageNode( LightBulb.WIRE_CURVE_IMAGE, mvt ) );
    addChild( new ModelElementImageNode( LightBulb.ELEMENT_BASE_BACK_IMAGE, mvt ) );
    var nonEnergizedBack = new ModelElementImageNode( FluorescentLightBulb.BACK_OFF, mvt );
    addChild( nonEnergizedBack );
    var energizedBack = new ModelElementImageNode( FluorescentLightBulb.BACK_ON, mvt );
    addChild( energizedBack );
    addChild( new EnergyChunkLayer( lightBulb.energyChunkList, lightBulb.getObservablePosition(), mvt ) );
    addChild( new ModelElementImageNode( LightBulb.ELEMENT_BASE_FRONT_IMAGE, mvt ) );
    var nonEnergizedFront = new ModelElementImageNode( FluorescentLightBulb.FRONT_OFF, mvt );
    addChild( nonEnergizedFront );
    var energizedFront = new ModelElementImageNode( FluorescentLightBulb.FRONT_ON, mvt );
    addChild( energizedFront );
    // Make bulb partially transparent when energy chunks visible.
    energyChunksVisible.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( visible ) {
        var opaqueness = visible ? 0.7f
        :
        1.0
        f;
        nonEnergizedFront.setTransparency( opaqueness );
        nonEnergizedBack.setTransparency( opaqueness );
      }
    } ) );
    // Center the light rays on the bulb image.
    lightRays.setOffset( energizedFront.getFullBoundsReference().getCenterX(), energizedFront.getFullBoundsReference().getCenterY() - energizedFront.getFullBoundsReference().getHeight() * 0.10 );
    // Update the transparency of the lit bulb based on model element.
    lightBulb.litProportion.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( litProportion ) {
        var opaqueness = energyChunksVisible.get() ? litProportion.floatValue() * 0.7f
        :
        litProportion.floatValue();
        energizedFront.setTransparency( opaqueness );
        energizedBack.setTransparency( opaqueness );
        lightRays.setTransparency( opaqueness );
      }
    } ) );
  }

  return inherit( PositionableFadableModelElementNode, FluorescentLightBulbNode, {
  } );
} );

