// Copyright 2002-2012, University of Colorado
/**
 * Base class for PNodes that represent energy system elements in the view
 * that use images as part of their representation.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EnergySystemElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/EnergySystemElement' );
  var ModelElementImage = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/ModelElementImage' );
  var PImage = require( 'edu.umd.cs.piccolo.nodes.PImage' );

  function ImageBasedEnergySystemElementNode( element, mvt ) {

    //private
    this.mvt;
    PositionableFadableModelElementNode.call( this, element, mvt );
    this.mvt = mvt;
  }

  return inherit( PositionableFadableModelElementNode, ImageBasedEnergySystemElementNode, {
    addImageNode: function( modelElementImage ) {
      var imageNode = new PImage( modelElementImage.getImage() );
      var widthInView = mvt.modelToViewDeltaX( modelElementImage.getWidth() );
      imageNode.setScale( widthInView / imageNode.getFullBoundsReference().width );
      var centerToCenterOffsetInView = mvt.modelToViewDelta( modelElementImage.getCenterToCenterOffset() ).toPoint2D();
      imageNode.centerFullBoundsOnPoint( centerToCenterOffsetInView.getX(), centerToCenterOffsetInView.getY() );
      addChild( imageNode );
      return imageNode;
    }
  } );
} );

