// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents the incandescent light bulb in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Vector2 = require( 'DOT/Vector2' );
  var Property = require( 'AXON/Property' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EnergyChunkLayer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkLayer' );
  var IncandescentLightBulb = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/IncandescentLightBulb' );
  var LightBulb = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/LightBulb' );
  var Node = require( 'SCENERY/nodes/Node' );

  function IncandescentLightBulbNode( lightBulb, energyChunksVisible, mvt ) {
    ImageBasedEnergySystemElementNode.call( this, lightBulb, mvt );
    // Add the light rays.
    var lightRays = new LightRays( new Vector2( 0, 0 ), 30, 400, 20, Math.PI / 4, Color.YELLOW );
    addChild( lightRays );
    energyChunksVisible.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( energyChunksVisible ) {
        // Only show the light rays when the energy chunks are not shown.
        lightRays.setVisible( !energyChunksVisible );
      }
    } ) );
    // Add the images and the layer that will contain the energy chunks.
    addImageNode( LightBulb.WIRE_FLAT_IMAGE );
    addImageNode( LightBulb.WIRE_CURVE_IMAGE );
    addImageNode( LightBulb.ELEMENT_BASE_BACK_IMAGE );
    addChild( new EnergyChunkLayer( lightBulb.energyChunkList, lightBulb.getObservablePosition(), mvt ) );
    addImageNode( LightBulb.ELEMENT_BASE_FRONT_IMAGE );
    addImageNode( IncandescentLightBulb.NON_ENERGIZED_BULB );
    var energizedBulb = addImageNode( IncandescentLightBulb.ENERGIZED_BULB );
    // Center the light rays on the bulb image.
    lightRays.setOffset( energizedBulb.getFullBoundsReference().getCenterX(), energizedBulb.getFullBoundsReference().getCenterY() - energizedBulb.getFullBoundsReference().getHeight() * 0.10 );
    // Update the transparency of the lit bulb based on model element.
    lightBulb.litProportion.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( litProportion ) {
        energizedBulb.setTransparency( litProportion.floatValue() );
        lightRays.setTransparency( litProportion.floatValue() );
      }
    } ) );
  }

  return inherit( ImageBasedEnergySystemElementNode, IncandescentLightBulbNode, {
  } );
} );

