// Copyright 2002-2012, University of Colorado
/**
 * Class that represents rays of light emitting from a light source.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var Vector2 = require( 'DOT/Vector2' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var Node = require( 'SCENERY/nodes/Node' );

// For debug.

  //private
  var SHOW_RAY_BLOCKING_SHAPES = false;

  function LightRays( center, innerRadius, outerRadius, numRays, color ) {

    //private
    this.lightRayNodes = [];
    this( center, innerRadius, outerRadius, numRays, 0, color );
  }

  function LightRays( center, innerRadius, outerRadius, numRaysForFullCircle, darkConeSpanAngle, color ) {

    //private
    this.lightRayNodes = [];
    // Parameter checking.
    assert && assert( numRaysForFullCircle > 0 && outerRadius > innerRadius && darkConeSpanAngle < Math.PI * 2 );
    // Create and add the rays.
    for ( var i = 0; i < numRaysForFullCircle; i++ ) {
      var angle = (Math.PI * 2 / numRaysForFullCircle) * i;
      var rayStartPoint = center.plus( new Vector2( innerRadius, 0 ).getRotatedInstance( angle ) );
      var rayEndPoint = center.plus( new Vector2( outerRadius, 0 ).getRotatedInstance( angle ) );
      if ( angle <= Math.PI / 2 - darkConeSpanAngle / 2 || angle >= Math.PI / 2 + darkConeSpanAngle / 2 ) {
        // Ray is not in the "dark cone", so add it.
        var lightRayNode = new LightRayNode( rayStartPoint, rayEndPoint, color );
        lightRayNodes.add( lightRayNode );
        addChild( lightRayNode );
      }
    }
  }

  return inherit( Node, LightRays, {
    addLightAbsorbingShape: function( lightAbsorbingShape ) {
      for ( var lightRayNode in lightRayNodes ) {
        lightRayNode.addLightAbsorbingShape( lightAbsorbingShape );
      }
      // For debug: Show the ray-blocking shapes. Note that these don't get removed if shape gets removed.
      if ( SHOW_RAY_BLOCKING_SHAPES ) {
        addChild( new PhetPPath( lightAbsorbingShape.shape, Color.PINK, new BasicStroke( 2 ), Color.GREEN ) );
      }
    },
    removeLightAbsorbingShape: function( lightAbsorbingShape ) {
      for ( var lightRayNode in lightRayNodes ) {
        lightRayNode.removeLightAbsorbingShape( lightAbsorbingShape );
      }
    }
  } );
} );

