// Copyright 2002-2012, University of Colorado
/**
 * Base class for model elements whose position and opacity can change.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'DOT/Vector2' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PositionableFadableModelElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/PositionableFadableModelElement' );
  var Node = require( 'SCENERY/nodes/Node' );

  function PositionableFadableModelElementNode( modelElement, mvt ) {
    // Update the overall offset based on the model position.
    modelElement.getObservablePosition().addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( immutableVector2D ) {
        setOffset( mvt.modelToView( immutableVector2D ).toPoint2D() );
      }
    } ) );
    // Update the overall opacity base on model element opacity.
    modelElement.opacity.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( opacity ) {
        setTransparency( opacity.floatValue() );
      }
    } ) );
  }

  return inherit( Node, PositionableFadableModelElementNode, {
  } );
} );

