// Copyright 2002-2012, University of Colorado
/**
 * Piccolo node that represents a drop of water in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Ellipse2D = require( 'java.awt.geom.Ellipse2D' );
  var Vector2 = require( 'DOT/Vector2' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var WaterDrop = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/model/WaterDrop' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PPath = require( 'edu.umd.cs.piccolo.nodes.PPath' );

  function WaterDropNode( waterDrop, mvt ) {
    // Create and maintain the initial shape.
    var waterDropNode = new PhetPPath( EFACConstants.WATER_COLOR_OPAQUE );
    waterDrop.size.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( dropSize ) {
        var dropWidth = mvt.modelToViewDeltaX( dropSize.getWidth() );
        var dropHeight = -mvt.modelToViewDeltaY( dropSize.getHeight() );
        waterDropNode.setPathTo( new Ellipse2D.Number( -dropWidth / 2, -dropHeight / 2, dropWidth, dropHeight ) );
      }
    } ) );
    addChild( waterDropNode );
    // Update offset as it changes.
    waterDrop.offsetFromParent.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( offsetFromParent ) {
        setOffset( mvt.modelToViewDeltaX( offsetFromParent.getX() ), mvt.modelToViewDeltaY( offsetFromParent.getY() ) );
      }
    } ) );
  }

  return inherit( Node, WaterDropNode, {
  } );
} );

