// Copyright 2002-2012, University of Colorado
/**
 * Developer control dialog used for experimenting with heat transfer values.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Font = require( 'SCENERY/util/Font' );
  var Frame = require( 'java.awt.Frame' );
  var GridLayout = require( 'java.awt.GridLayout' );
  var ActionEvent = require( 'java.awt.event.ActionEvent' );
  var ActionListener = require( 'java.awt.event.ActionListener' );
  var DecimalFormat = require( 'java.text.DecimalFormat' );
  var Format = require( 'java.text.Format' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var JButton = require( 'javax.swing.JButton' );
  var JLabel = require( 'javax.swing.JLabel' );
  var JTextField = require( 'javax.swing.JTextField' );
  var PaintImmediateDialog = require( 'edu.colorado.phet.common.phetcommon.application.PaintImmediateDialog' );
  var Property = require( 'AXON/Property' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var PhetFont = require( 'edu.colorado.phet.common.phetcommon.view.util.PhetFont' );
  var HeatTransferConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/HeatTransferConstants' );


  // static class: ParameterEntryField
  var ParameterEntryField =
// Convenience class for linking parameter entry to a property.

    //private
    define( function( require ) {

      //private
      var LABEL_FONT = new PhetFont( 14 );

      //private
      var FORMATTER = new DecimalFormat( "###.0#" );

      function ParameterEntryField( labelText, property ) {

        //private
        this.label;

        //private
        this.textField;

        //private
        this.property;
        label = new JLabel( labelText );
        label.setFont( LABEL_FONT );
        this.property = property;
        textField = new JTextField();
        textField.setFont( LABEL_FONT );
        property.addObserver( new VoidFunction1().withAnonymousClassBody( {
          apply: function( value ) {
            textField.setText( FORMATTER.format( value ) );
          }
        } ) );
        textField.setText( FORMATTER.format( property.get() ) );
        textField.addActionListener( new ActionListener().withAnonymousClassBody( {
          actionPerformed: function( e ) {
            console.log( "Action Performed in HeatTransferValuesDialog" );
            property.set( Number.parseDouble( textField.getText() ) );
          }
        } ) );
      }

      return inherit( Object, ParameterEntryField, {
        getLabel: function() {
          return label;
        },
        getTextField: function() {
          return textField;
        },
        getProperty: function() {
          return property;
        }
      } );
    } );
  ;
  function HeatTransferValuesDialog( frame ) {
    PaintImmediateDialog.call( this, frame );
    setTitle( "Heat Transfer Values" );
    setLayout( new GridLayout( 8, 2 ) );
    // Create a list of the parameters that can be changes.
    var parameterEntryFields = [].withAnonymousClassBody( {
      initializer: function() {
        add( new ParameterEntryField( "Brick-Iron:", HeatTransferConstants.BRICK_IRON_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Brick-Water:", HeatTransferConstants.BRICK_WATER_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Brick-Air:", HeatTransferConstants.BRICK_AIR_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Iron-Water:", HeatTransferConstants.IRON_WATER_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Iron-Air:", HeatTransferConstants.IRON_AIR_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Water-Air:", HeatTransferConstants.WATER_AIR_HEAT_TRANSFER_FACTOR ) );
        add( new ParameterEntryField( "Air-Surrounding Air:", HeatTransferConstants.AIR_TO_SURROUNDING_AIR_HEAT_TRANSFER_FACTOR ) );
      }
    } );
    // Add the parameters to the dialog.
    for ( var parameterEntryField in parameterEntryFields ) {
      add( parameterEntryField );
    }
    // Add a button for resetting to default values.
    var resetButton = new JButton( "Reset" );
    resetButton.addActionListener( new ActionListener().withAnonymousClassBody( {
      actionPerformed: function( e ) {
        for ( var parameterEntryField in parameterEntryFields ) {
          parameterEntryField.getProperty().reset();
        }
      }
    } ) );
    add( resetButton );
    pack();
  }

  return inherit( PaintImmediateDialog, HeatTransferValuesDialog, {

    //private
    add: function( parameterEntryField ) {
      add( parameterEntryField.getLabel() );
      add( parameterEntryField.getTextField() );
    },
  } );
} );

