// Copyright 2002-2012, University of Colorado
/**
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Image = require( 'java.awt.Image' );
  var Vector2 = require( 'DOT/Vector2' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );

// In J/kg-K, source = design document.

  //private
  var SPECIFIC_HEAT = 840;
// In kg/m^3, source = design document plus some tweaking to keep chunk numbers reasonable.

  //private
  var DENSITY = 3300;
// Some constants needed for energy chunk mapping.
// In joules.
  var ENERGY_AT_ROOM_TEMPERATURE = Math.pow( SURFACE_WIDTH, 3 ) * DENSITY * SPECIFIC_HEAT * EFACConstants.ROOM_TEMPERATURE;
// In joules.
  var ENERGY_AT_WATER_FREEZING_TEMPERATURE = Math.pow( SURFACE_WIDTH, 3 ) * DENSITY * SPECIFIC_HEAT * EFACConstants.FREEZING_POINT_TEMPERATURE;

  function Brick( clock, initialPosition, energyChunksVisible ) {
    Block.call( this, clock, initialPosition, DENSITY, SPECIFIC_HEAT, energyChunksVisible );
  }

  return inherit( Block, Brick, {
      getFrontTextureImage: function() {
        return EnergyFormsAndChangesResources.Images.BRICK_TEXTURE_FRONT;
      },
      getTopTextureImage: function() {
        return EnergyFormsAndChangesResources.Images.BRICK_TEXTURE_TOP;
      },
      getSideTextureImage: function() {
        return EnergyFormsAndChangesResources.Images.BRICK_TEXTURE_RIGHT;
      },
      getColor: function() {
        return new Color( 200, 22, 11 );
      },
      getLabel: function() {
        return EnergyFormsAndChangesResources.Strings.BRICK;
      },
      getEnergyContainerCategory: function() {
        return EnergyContainerCategory.BRICK;
      },
      getUserComponent: function() {
        return EnergyFormsAndChangesSimSharing.UserComponents.brick;
      }
    },
//statics
    {
      ENERGY_AT_ROOM_TEMPERATURE: ENERGY_AT_ROOM_TEMPERATURE,
      ENERGY_AT_WATER_FREEZING_TEMPERATURE: ENERGY_AT_WATER_FREEZING_TEMPERATURE
    } );
} );

