// Copyright 2002-2012, University of Colorado
/**
 * Class that represents a block whose specific heat can be changed.
 * <p/>
 * HCL: This was created for heat capacity lab prototype, and has been removed
 * as of 8/6/2012.  It was decided in March 2013 to retain this and the other
 * HCL in the code base in case it is needed again.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var MathUtil = require( 'edu.colorado.phet.common.phetcommon.math.MathUtil' );
  var Vector2 = require( 'DOT/Vector2' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var ChangeObserver = require( 'edu.colorado.phet.common.phetcommon.model.property.ChangeObserver' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var ColorUtils = require( 'edu.colorado.phet.common.phetcommon.view.util.ColorUtils' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );

//-------------------------------------------------------------------------
// Class Data
//-------------------------------------------------------------------------

  //private
  var MIN_SPECIFIC_HEAT = 840;

  //private
  var MAX_SPECIFIC_HEAT = 3000;
// In J/kg-K, source = design document.

  //private
  var INITIAL_SPECIFIC_HEAT = MIN_SPECIFIC_HEAT;
// In kg/m^3.

  //private
  var DENSITY = 2000;

  //private
  var LOW_SPECIFIC_HEAT_COLOR = new Color( 142, 107, 35 );

  //private
  var HIGH_SPECIFIC_HEAT_COLOR = new Color( 255, 185, 15 );

  function ConfigurableHeatCapacityBlock( clock, initialPosition, energyChunksVisible ) {
    //-------------------------------------------------------------------------
    // Instance Data
    //-------------------------------------------------------------------------
    // Specific heat, which is configurable in this block.

    //private
    this.specificHeat = new Property( INITIAL_SPECIFIC_HEAT );
    // Color changes as specific heat changes.
    this.color = new Property( mapSpecificHeatToColor( INITIAL_SPECIFIC_HEAT ) );
    Block.call( this, clock, initialPosition, DENSITY, INITIAL_SPECIFIC_HEAT, energyChunksVisible );
    specificHeat.addObserver( new ChangeObserver().withAnonymousClassBody( {
      update: function( newSpecificHeat, oldSpecificHeat ) {
        // Bounds checking.
        assert && assert( newSpecificHeat >= MIN_SPECIFIC_HEAT && newSpecificHeat <= MAX_SPECIFIC_HEAT );
        // Set the color based on the specific heat value.
        color.set( mapSpecificHeatToColor( newSpecificHeat ) );
        // Add or remove energy in order to keep the temperature constant.
        var oldTemperature = getEnergy() / (mass * oldSpecificHeat);
        var newTemperature = getEnergy() / (mass * newSpecificHeat);
        changeEnergy( (oldTemperature - newTemperature) * mass * specificHeat.get() );
        // vanish as needed.
        while ( getEnergyChunkBalance() != 0 ) {
          if ( getEnergyChunkBalance() > 0 ) {
            extractClosestEnergyChunk( getCenterPoint() );
          }
          else if ( getEnergyChunkBalance() < 0 ) {
            addEnergyChunk( new EnergyChunk( EnergyType.THERMAL, getCenterPoint(), energyChunksVisible ) );
          }
        }
      }
    } ) );
  }

  return inherit( Block, ConfigurableHeatCapacityBlock, {
//-------------------------------------------------------------------------
// Methods
//-------------------------------------------------------------------------
    getColor: function() {
      return color.get();
    },
    getLabel: function() {
      return "Custom";
    },
    getEnergyContainerCategory: function() {
      // permanent, factors for this block may be needed.
      return EnergyContainerCategory.BRICK;
    },
    getUserComponent: function() {
      return EnergyFormsAndChangesSimSharing.UserComponents.brick;
    },
    reset: function() {
      specificHeat.reset();
      super.reset();
    },
    getTemperature: function() {
      return energy / (mass * specificHeat.get());
    },

    //private
    mapSpecificHeatToColor: function( specificHeat ) {
      var proportion = MathUtil.clamp( 0, (specificHeat - MIN_SPECIFIC_HEAT) / (MAX_SPECIFIC_HEAT - MIN_SPECIFIC_HEAT), 1 );
      return ColorUtils.interpolateRBGA( LOW_SPECIFIC_HEAT_COLOR, HIGH_SPECIFIC_HEAT_COLOR, proportion );
    }
  } );
} );

