// Copyright 2002-2012, University of Colorado
/**
 * Class that represents a thermometer that can stick to other elements as
 * they move.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'DOT/Vector2' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var Thermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Thermometer' );


  // static class: ElementFollower
  var ElementFollower =
// Convenience class for sticking to model elements.

    //private
    define( function( require ) {

      //private
      function ElementFollower( follower ) {

        //private
        this.follower;

        //private
        this.locationBeingFollowed = null;

        //private
        this.offset = new Vector2( 0, 0 );

        //private
        this.followerFunction = new VoidFunction1().withAnonymousClassBody( {
          apply: function( location ) {
            follower.set( location.plus( offset ) );
          }
        } );
        this.follower = follower;
      }

      return inherit( Object, ElementFollower, {
        follow: function( locationToFollow ) {
          if ( locationBeingFollowed != null ) {
            locationBeingFollowed.removeObserver( followerFunction );
          }
          offset = follower.get().minus( locationToFollow.get() );
          locationToFollow.addObserver( followerFunction );
          locationBeingFollowed = locationToFollow;
        },
        stopFollowing: function() {
          if ( locationBeingFollowed != null ) {
            locationBeingFollowed.removeObserver( followerFunction );
            locationBeingFollowed = null;
          }
        },
        isFollowing: function() {
          return locationBeingFollowed != null;
        }
      } );
    } );
  ;
  function ElementFollowingThermometer( model, initialPosition, initiallyActive ) {

    //private
    this.elementFollower = new ElementFollower( this.position );
    Thermometer.call( this, model.getClock(), model, initialPosition, initiallyActive );
    // anything that it should stick to.
    userControlled.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( userControlled ) {
        if ( userControlled ) {
          // Stop following anything.
          elementFollower.stopFollowing();
        }
        else {
          // dropped over something that it should follow.
          for ( var block in model.getBlockList() ) {
            if ( block.getProjectedShape().contains( position.get().toPoint2D() ) ) {
              // Stick to this block.
              elementFollower.follow( block.position );
            }
          }
          if ( !elementFollower.isFollowing() && model.getBeaker().getThermalContactArea().getBounds().contains( position.get().toPoint2D() ) ) {
            // Stick to the beaker.
            elementFollower.follow( model.getBeaker().position );
          }
        }
      }
    } ) );
  }

  return inherit( Thermometer, ElementFollowingThermometer, {
    reset: function() {
      elementFollower.stopFollowing();
      super.reset();
    },
  } );
} );

