// Copyright 2002-2012, University of Colorado
/**
 * This class represents a "slice" within a 2D container that contains a set of
 * energy chunks and can be used to add some limited 3D capabilities by having
 * some z-dimension information.  The slice consists of a 2D shape and a Z
 * value representing its position in Z space.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Shape = require( 'java.awt.Shape' );
  var AffineTransform = require( 'java.awt.geom.AffineTransform' );
  var Vector2 = require( 'DOT/Vector2' );
  var ChangeObserver = require( 'edu.colorado.phet.common.phetcommon.model.property.ChangeObserver' );
  var Property = require( 'AXON/Property' );
  var ObservableArray = require( 'AXON/ObservableArray' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );

  function EnergyChunkContainerSlice( shape, zPosition, anchorPoint ) {
    this.energyChunkList = new ObservableArray();

    //private
    this.zPosition;

    //private
    this.shape;
    this.shape = shape;
    this.zPosition = zPosition;
    // Monitor the anchor position and move the contained energy chunks to match.
    anchorPoint.addObserver( new ChangeObserver().withAnonymousClassBody( {
      update: function( newPosition, oldPosition ) {
        var translation = newPosition.minus( oldPosition );
        EnergyChunkContainerSlice.this.shape = AffineTransform.getTranslateInstance( translation.getX(), translation.getY() ).createTransformedShape( EnergyChunkContainerSlice.this.shape );
        for ( var energyChunk in energyChunkList ) {
          energyChunk.translate( translation );
        }
      }
    } ) );
  }

  return inherit( Object, EnergyChunkContainerSlice, {
    addEnergyChunk: function( energyChunk ) {
      energyChunk.zPosition.set( zPosition );
      energyChunkList.add( energyChunk );
    },
    getNumEnergyChunks: function() {
      return energyChunkList.size();
    },
    setShape: function( shape ) {
      this.shape = shape;
    },
    getShape: function() {
      return shape;
    }
  } );
} );

