// Copyright 2002-2012, University of Colorado
/**
 * A node that represents a 2D surface on which energy chunks reside.  The
 * surface contains z-dimension information, and can thus be used to create an
 * effect of layering in order to get a bit of a 3D appearance.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyChunkNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkNode' );
  var Node = require( 'SCENERY/nodes/Node' );


  //private
  var ALWAYS_SHOW_OUTLINE = false;

  function EnergyChunkContainerSliceNode( energyChunkContainerSlice, mvt ) {
    this( energyChunkContainerSlice, mvt, false, Color.RED );
  }

  function EnergyChunkContainerSliceNode( energyChunkContainerSlice, mvt, showOutline, outlineColor ) {
    // Watch for energy chunks coming and going and add/remove nodes accordingly.
    energyChunkContainerSlice.energyChunkList.addElementAddedObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( addedEnergyChunk ) {
        var energyChunkNode = new EnergyChunkNode( addedEnergyChunk, mvt );
        addChild( energyChunkNode );
        energyChunkContainerSlice.energyChunkList.addElementRemovedObserver( new VoidFunction1().withAnonymousClassBody( {
          apply: function( removedEnergyChunk ) {
            if ( removedEnergyChunk == addedEnergyChunk ) {
              removeChild( energyChunkNode );
              energyChunkContainerSlice.energyChunkList.removeElementRemovedObserver( this );
            }
          }
        } ) );
      }
    } ) );
    // For debug.
    if ( showOutline || ALWAYS_SHOW_OUTLINE ) {
      addChild( new PhetPPath( mvt.modelToView( energyChunkContainerSlice.getShape() ), new BasicStroke( 1 ), outlineColor ) );
    }
  }

  return inherit( Node, EnergyChunkContainerSliceNode, {
  } );
} );

