// Copyright 2002-2012, University of Colorado
/**
 * A simple horizontal surface.  This is represented by a range of x values
 * and a single y value.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var DoubleRange = require( 'edu.colorado.phet.common.phetcommon.util.DoubleRange' );

  function HorizontalSurface( xRange, yPos, owner ) {
    this.xRange;
    this.yPos;

    //private
    this.elementOnSurface = null;

    //private
    this.owner;
    this.xRange = xRange;
    this.yPos = yPos;
    this.owner = owner;
  }

  return inherit( Object, HorizontalSurface, {
    overlapsWith: function( surface ) {
      return (xRange.intersectsExclusive( surface.xRange ));
    },
    getCenterX: function() {
      return xRange.getCenter();
    },
    getOwner: function() {
      return owner;
    },
    getElementOnSurface: function() {
      return elementOnSurface;
    },
    addElementToSurface: function( modelElement ) {
      // Only one thing on surface allowed at a time.
      assert && assert( elementOnSurface == null );
      elementOnSurface = modelElement;
    },
    clearSurface: function() {
      elementOnSurface = null;
    },
    equals: function( o ) {
      if ( this == o ) {
        return true;
      }
      if ( o == null || getClass() != o.getClass() ) {
        return false;
      }
      var that = o;
      return Number.compare( that.yPos, yPos ) == 0 && !(xRange != null ? !xRange.equals( that.xRange ) : that.xRange != null);
    },
    hashCode: function() {
      var result;
      var temp;
      result = xRange != null ? xRange.hashCode() : 0;
      temp = yPos != +0.0
      d ? Number.doubleToLongBits( yPos ) : 0
      L;
      result = 31 * result + (temp ^ (temp >>> 32));
      return result;
    }
  } );
} );

