// Copyright 2002-2012, University of Colorado
/**
 * Class that represents a block of iron in the view.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Vector2 = require( 'DOT/Vector2' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );

// In J/kg-K

  //private
  var SPECIFIC_HEAT = 450;
//    private static final double SPECIFIC_HEAT = 275; // In J/kg-K
// In kg/m^3, source = design document.

  //private
  var DENSITY = 7800;

  function IronBlock( clock, initialPosition, energyChunksVisible ) {
    Block.call( this, clock, initialPosition, DENSITY, SPECIFIC_HEAT, energyChunksVisible );
  }

  return inherit( Block, IronBlock, {
    getColor: function() {
      return new Color( 150, 150, 150 );
    },
    getLabel: function() {
      return EnergyFormsAndChangesResources.Strings.IRON;
    },
    getEnergyContainerCategory: function() {
      return EnergyContainerCategory.IRON;
    },
    getUserComponent: function() {
      return EnergyFormsAndChangesSimSharing.UserComponents.ironBlock;
    }
  } );
} );

