// Copyright 2002-2012, University of Colorado
/**
 * Base class for all model elements in this module.  At the time of this
 * writing, this includes blocks, beakers, burners, and thermometers.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );


  return inherit( Object, ModelElement, {
    /**
     * Get the top surface of this model element.  Only model elements that can
     * support other elements on top of them have top surfaces.
     *
     * @return The top surface of this model element, null if this element can
     *         not have other elements on top of it.
     */
    getTopSurfaceProperty: function() {
      // Defaults to null, override as needed.
      return null;
    },
    /**
     * Get the bottom surface of this model element.  Only model elements that
     * can rest on top of other model elements have bottom surfaces.
     *
     * @return The bottom surface of this model element, null if this element
     *         never rests upon other model elements.
     */
    getBottomSurfaceProperty: function() {
      // Defaults to null, override as needed.
      return null;
    },
    /**
     * Get the surface upon which this model element is resting, if there is
     * one.
     *
     * @return Surface upon which this element is resting, null if there is
     *         none.
     */
    getSupportingSurface: function() {
      return supportingSurface;
    },
    /*
     * Set the surface upon which this model element is resting.
     */
    setSupportingSurface: function( surfaceProperty ) {
      supportingSurface = surfaceProperty;
    },
    /**
     * Get a value that indicates whether this element is stacked upon the
     * given model element.
     *
     * @param element Model element to be checked.
     * @return true if the given element is stacked anywhere on top of this
     *         one, which includes cases where one or more elements are in between.
     */
    isStackedUpon: function( element ) {
      return getSupportingSurface() != null && (getSupportingSurface().get().getOwner() == element || getSupportingSurface().get().getOwner().isStackedUpon( element ));
    },
    /**
     * Reset the model element to its original state.  Subclasses must add
     * reset functionality for any state that they add.
     */
    reset: function() {
      if ( supportingSurface != null ) {
        supportingSurface.removeAllObservers();
        supportingSurface.get().clearSurface();
        supportingSurface = null;
      }
    }
  } );
} );

