// Copyright 2002-2012, University of Colorado
/**
 * Base class for model elements that can be moved around by the user.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'DOT/Vector2' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );
  var IUserComponent = require( 'edu.colorado.phet.common.phetcommon.simsharing.messages.IUserComponent' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );

  /**
   * Constructor.
   */
  function UserMovableModelElement( initialPosition ) {
    this.userControlled = new BooleanProperty( false );
    // Position of the center of the bottom of the block.
    this.position;
    // Velocity in the up/down direction.
    this.verticalVelocity = new Property( 0.0 );
    // Observer that moves this model element if an when the surface that is
    // supporting it moves.

    //private
    this.surfaceMotionObserver = new VoidFunction1().withAnonymousClassBody( {
      apply: function( horizontalSurface ) {
        var value = new Vector2( horizontalSurface.getCenterX(), horizontalSurface.yPos );
        position.set( value );
      }
    } );
    position = new Property( initialPosition );
    userControlled.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( userControlled ) {
        if ( userControlled ) {
          // longer sitting on any surface.
          if ( getSupportingSurface() != null ) {
            getSupportingSurface().removeObserver( surfaceMotionObserver );
            getSupportingSurface().get().clearSurface();
            setSupportingSurface( null );
          }
        }
      }
    } ) );
  }

  return inherit( ModelElement, UserMovableModelElement, {
    setSupportingSurface: function( surfaceProperty ) {
      super.setSupportingSurface( surfaceProperty );
      if ( surfaceProperty != null ) {
        surfaceProperty.addObserver( surfaceMotionObserver );
      }
    },
    reset: function() {
      if ( getSupportingSurface() != null ) {
        getSupportingSurface().removeObserver( surfaceMotionObserver );
      }
      userControlled.reset();
      position.reset();
      verticalVelocity.reset();
      super.reset();
    },
    /**
     * Get the "user component" identifier.  This supports the sim sharing
     * feature.
     *
     * @return user component identifier.
     */
    getUserComponent: function() {},
    getBottomSurfaceProperty: function() {},
    setX: function( x ) {
      position.set( new Vector2( x, position.get().getY() ) );
    }
  } );
} );

