// Copyright 2002-2012, University of Colorado
/**
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var BasicStroke = require( 'java.awt.BasicStroke' );
  var Color = require( 'SCENERY/util/Color' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyChunkNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkNode' );
  var Air = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/Air' );
  var Node = require( 'SCENERY/nodes/Node' );


  //private
  var SHOW_BOUNDS = false;

  function AirNode( air, mvt ) {
    if ( SHOW_BOUNDS ) {
      addChild( new PhetPPath( mvt.modelToView( air.getThermalContactArea().getBounds() ), new BasicStroke( 1 ), Color.RED ) );
    }
    // Create a layer where energy chunks will be placed.
    var energyChunkLayer = new Node();
    addChild( energyChunkLayer );
    // Watch for energy chunks coming and going and add/remove nodes accordingly.
    air.getEnergyChunkList().addElementAddedObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( addedEnergyChunk ) {
        var energyChunkNode = new EnergyChunkNode( addedEnergyChunk, mvt );
        energyChunkLayer.addChild( energyChunkNode );
        air.getEnergyChunkList().addElementRemovedObserver( new VoidFunction1().withAnonymousClassBody( {
          apply: function( removedEnergyChunk ) {
            if ( removedEnergyChunk == addedEnergyChunk ) {
              energyChunkLayer.removeChild( energyChunkNode );
              air.getEnergyChunkList().removeElementRemovedObserver( this );
            }
          }
        } ) );
      }
    } ) );
  }

  return inherit( Node, AirNode, {
  } );
} );

