// Copyright 2002-2012, University of Colorado
/**
 * Piccolo canvas for the "Intro" tab of the Energy Forms and Changes
 * simulation.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var Point = require( 'java.awt.Point' );
  var Shape = require( 'java.awt.Shape' );
  var ActionEvent = require( 'java.awt.event.ActionEvent' );
  var ActionListener = require( 'java.awt.event.ActionListener' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Rectangle = require( 'KITE/Rectangle' );
  var ArrayList = require( 'java.util.ArrayList' );
  var Resettable = require( 'edu.colorado.phet.common.phetcommon.model.Resettable' );
  var ConstantDtClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.ConstantDtClock' );
  var Property = require( 'AXON/Property' );
  var SimpleObserver = require( 'edu.colorado.phet.common.phetcommon.util.SimpleObserver' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var PropertyCheckBox = require( 'edu.colorado.phet.common.phetcommon.view.controls.PropertyCheckBox' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetFont = require( 'edu.colorado.phet.common.phetcommon.view.util.PhetFont' );
  var PhetPCanvas = require( 'edu.colorado.phet.common.piccolophet.PhetPCanvas' );
  var CursorHandler = require( 'edu.colorado.phet.common.piccolophet.event.CursorHandler' );
  var ControlPanelNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ControlPanelNode' );
  var PhetPPath = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPPath' );
  var PhetPText = require( 'edu.colorado.phet.common.piccolophet.nodes.PhetPText' );
  var ResetAllButtonNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ResetAllButtonNode' );
  var TextButtonNode = require( 'edu.colorado.phet.common.piccolophet.nodes.TextButtonNode' );
  var HBox = require( 'edu.colorado.phet.common.piccolophet.nodes.layout.HBox' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var EnergyFormsAndChangesSimSharing = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var Beaker = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Beaker' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );
  var BeakerView = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/BeakerView' );
  var BurnerStandNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/BurnerStandNode' );
  var EnergyChunkNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkNode' );
  var HeaterCoolerView = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/energysystems/view/HeaterCoolerView' );
  var EFACIntroModel = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/EFACIntroModel' );
  var ElementFollowingThermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/ElementFollowingThermometer' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PBasicInputEventHandler = require( 'edu.umd.cs.piccolo.event.PBasicInputEventHandler' );
  var PInputEvent = require( 'edu.umd.cs.piccolo.event.PInputEvent' );
  var PImage = require( 'edu.umd.cs.piccolo.nodes.PImage' );
  var PSwing = require( 'edu.umd.cs.piccolox.pswing.PSwing' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );//static ///*

  var STAGE_SIZE = CenteredStage.DEFAULT_STAGE_SIZE;

  //private
  var EDGE_INSET = 10;
// Multiplier empirically determined for best look.
  var BURNER_EDGE_TO_HEIGHT_RATIO = 0.2;
// Boolean property for showing/hiding developer control for dumping energy levels.
  var showDumpEnergiesButton = new BooleanProperty( false );

  /**
   * Constructor.
   *
   * @param model Model being portrayed on this canvas.
   */
  function EFACIntroCanvas( model ) {

    //private
    this.model;

    //private
    this.thermometerToolBox;

    //private
    this.normalSimSpeed = new BooleanProperty( true );
    this.model = model;
    // Set up the canvas-screen transform.
    setWorldTransformStrategy( new CenteredStage( this ) );
    // ones zoom in).
    var mvt = ModelViewTransform.createSinglePointScaleInvertedYMapping( new Vector2( 0, 0 ), new Point( Math.round( STAGE_SIZE.getWidth() * 0.5 ), Math.round( STAGE_SIZE.getHeight() * 0.85 ) ), // "Zoom factor" - smaller zooms out, larger zooms in.
      2200 );
    setBackground( FIRST_TAB_BACKGROUND_COLOR );
    // Set up a root node for our scene graph.
    var rootNode = new Node();
    addWorldChild( rootNode );
    // needed Z-order behavior.
    var backLayer = new Node();
    rootNode.addChild( backLayer );
    var beakerBackLayer = new Node();
    rootNode.addChild( beakerBackLayer );
    var beakerGrabLayer = new Node();
    rootNode.addChild( beakerGrabLayer );
    var blockLayer = new Node();
    rootNode.addChild( blockLayer );
    var airLayer = new Node();
    rootNode.addChild( airLayer );
    var heaterCoolerFrontLayer = new Node();
    rootNode.addChild( heaterCoolerFrontLayer );
    var thermometerLayer = new Node();
    rootNode.addChild( thermometerLayer );
    var beakerFrontLayer = new Node();
    rootNode.addChild( beakerFrontLayer );
    // Add the lab bench surface.
    var labBenchSurface = new PImage( EnergyFormsAndChangesResources.Images.SHELF_LONG );
    labBenchSurface.setOffset( mvt.modelToViewX( 0 ) - labBenchSurface.getFullBoundsReference().getWidth() / 2, // Slight tweak factor here due to nature of image.
        mvt.modelToViewY( 0 ) - labBenchSurface.getFullBoundsReference().getHeight() / 2 + 10 );
    backLayer.addChild( labBenchSurface );
    // Add a node that will act as the background below the lab bench
    // surface, basically like the side of the bench.
    {
      var width = labBenchSurface.getFullBoundsReference().getWidth() * 0.95;
      // Arbitrary large number, user should never see the bottom of this.
      var height = 1000;
      var benchSupportShape = new Rectangle.Number( labBenchSurface.getFullBoundsReference().getCenterX() - width / 2, labBenchSurface.getFullBoundsReference().getCenterY(), width, height );
      var labBenchSide = new PhetPPath( benchSupportShape, EFACConstants.CLOCK_CONTROL_BACKGROUND_COLOR );
      backLayer.addChild( labBenchSide );
      // Must be behind bench top.
      labBenchSide.moveToBack();
    }
    // the bench and the bottom of the canvas.  This is for layout.
    var centerYBelowSurface = (STAGE_SIZE.getHeight() + labBenchSurface.getFullBoundsReference().getMaxY()) / 2;
    // Add the clock controls.
    {
      var clockControl = new NormalAndFastForwardTimeControlPanel( normalSimSpeed, model.getClock() );
      clockControl.centerFullBoundsOnPoint( STAGE_SIZE.getWidth() / 2, centerYBelowSurface );
      normalSimSpeed.addObserver( new VoidFunction1().withAnonymousClassBody( {
        apply: function( normalSimSpeed ) {
          var clock = model.getClock();
          clock.setDt( normalSimSpeed ? SIM_TIME_PER_TICK_NORMAL : SIM_TIME_PER_TICK_FAST_FORWARD );
        }
      } ) );
      backLayer.addChild( clockControl );
    }
    // Add the reset button.
    {
      var resetButton = new ResetAllButtonNode( this, this, 20, Color.black, new Color( 255, 153, 0 ) );
      resetButton.setConfirmationEnabled( false );
      resetButton.setOffset( STAGE_SIZE.getWidth() - resetButton.getFullBoundsReference().width - 20, centerYBelowSurface - resetButton.getFullBoundsReference().getHeight() / 2 );
      backLayer.addChild( resetButton );
    }
    // Add the control for showing/hiding energy chunks.  The elements of
    // this control are created separately to allow each to be independently scaled.
    {
      var energyChunkNode = EnergyChunkNode.createEnergyChunkNode( EnergyType.THERMAL );
      energyChunkNode.setScale( 1.0 );
      energyChunkNode.setPickable( false );
      var label = new PhetPText( EnergyFormsAndChangesResources.Strings.ENERGY_SYMBOLS, new PhetFont( 20 ) ).withAnonymousClassBody( {
        initializer: function() {
          setPickable( false );
          setChildrenPickable( false );
        }
      } );
      var showEnergyCheckBox = new PropertyCheckBox( EnergyFormsAndChangesSimSharing.UserComponents.showEnergyCheckBox, null, model.energyChunksVisible );
      showEnergyCheckBox.setFont( new PhetFont( 40 ) );
      var checkBoxNode = new PSwing( showEnergyCheckBox ).withAnonymousClassBody( {
        initializer: function() {
          setScale( 1.5 );
          setPickable( false );
          setChildrenPickable( false );
        }
      } );
      var hBox = new HBox( 5, checkBoxNode, label, energyChunkNode );
      var controlPanel = new ControlPanelNode( hBox, CONTROL_PANEL_BACKGROUND_COLOR, CONTROL_PANEL_OUTLINE_STROKE, CONTROL_PANEL_OUTLINE_COLOR ).withAnonymousClassBody( {
        initializer: function() {
          setOffset( STAGE_SIZE.getWidth() - getFullBoundsReference().width - EDGE_INSET, EDGE_INSET );
          addInputEventListener( new PBasicInputEventHandler().withAnonymousClassBody( {
            mouseClicked: function( event ) {
              model.energyChunksVisible.set( !model.energyChunksVisible.get() );
            }
          } ) );
          addInputEventListener( new CursorHandler() );
        }
      } );
    }
    // Add developer control for printing out energy values.
    {
      var dumpEnergiesButton = new TextButtonNode( "Dump Energies", new PhetFont( 14 ) );
      dumpEnergiesButton.addActionListener( new ActionListener().withAnonymousClassBody( {
        actionPerformed: function( e ) {
          model.dumpEnergies();
        }
      } ) );
      dumpEnergiesButton.setOffset( 20, centerYBelowSurface - dumpEnergiesButton.getFullBoundsReference().getHeight() / 2 );
      backLayer.addChild( dumpEnergiesButton );
      // Control the visibility of this button.
      showDumpEnergiesButton.addObserver( new VoidFunction1().withAnonymousClassBody( {
        apply: function( visible ) {
          dumpEnergiesButton.setVisible( visible );
        }
      } ) );
    }
    // Add the burners.
    var burnerProjectionAmount = mvt.modelToView( model.getLeftBurner().getOutlineRect() ).getBounds2D().getWidth() * BURNER_EDGE_TO_HEIGHT_RATIO;
    var burnerWidth = mvt.modelToViewDeltaX( model.getLeftBurner().getOutlineRect().getWidth() ) * 0.7;
    var burnerHeight = burnerWidth * 0.8;
    var burnerOpeningHeight = burnerHeight * 0.2;
    // Empirically determined for best look.
    var burnerYPosTweak = -10;
    // Set up left heater-cooler node.
    var leftHeaterCooler = new HeaterCoolerView( model.getLeftBurner().heatCoolLevel, true, true, EnergyFormsAndChangesResources.Strings.HEAT, EnergyFormsAndChangesResources.Strings.COOL, burnerWidth, burnerHeight, burnerOpeningHeight, true, model.getLeftBurner().energyChunkList, mvt );
    leftHeaterCooler.setOffset( mvt.modelToViewX( model.getLeftBurner().getOutlineRect().getCenterX() ) - leftHeaterCooler.getHoleNode().getFullBounds().getWidth() / 2, mvt.modelToViewY( model.getLeftBurner().getOutlineRect().getMinY() ) - leftHeaterCooler.getFrontNode().getFullBounds().getHeight() - burnerYPosTweak );
    backLayer.addChild( leftHeaterCooler.getHoleNode() );
    backLayer.addChild( new BurnerStandNode( mvt.modelToView( model.getLeftBurner().getOutlineRect() ).getBounds2D(), burnerProjectionAmount ) );
    heaterCoolerFrontLayer.addChild( leftHeaterCooler.getFrontNode() );
    // Set up right heater-cooler node.
    var rightHeaterCooler = new HeaterCoolerView( model.getRightBurner().heatCoolLevel, true, true, EnergyFormsAndChangesResources.Strings.HEAT, EnergyFormsAndChangesResources.Strings.COOL, burnerWidth, burnerHeight, burnerOpeningHeight, true, model.getRightBurner().energyChunkList, mvt );
    rightHeaterCooler.setOffset( mvt.modelToViewX( model.getRightBurner().getOutlineRect().getCenterX() ) - rightHeaterCooler.getHoleNode().getFullBounds().getWidth() / 2, mvt.modelToViewY( model.getRightBurner().getOutlineRect().getMinY() ) - rightHeaterCooler.getFrontNode().getFullBounds().getHeight() - burnerYPosTweak );
    backLayer.addChild( rightHeaterCooler.getHoleNode() );
    backLayer.addChild( new BurnerStandNode( mvt.modelToView( model.getRightBurner().getOutlineRect() ).getBounds2D(), burnerProjectionAmount ) );
    heaterCoolerFrontLayer.addChild( rightHeaterCooler.getFrontNode() );
    // Add the air.
    airLayer.addChild( new AirNode( model.getAir(), mvt ) );
    // Add the movable objects.
    var brickNode = new BlockNode( model, model.getBrick(), mvt );
    brickNode.setApproachingEnergyChunkParentNode( airLayer );
    blockLayer.addChild( brickNode );
    var ironBlockNode = new BlockNode( model, model.getIronBlock(), mvt );
    ironBlockNode.setApproachingEnergyChunkParentNode( airLayer );
    blockLayer.addChild( ironBlockNode );
    var beakerView = new BeakerContainerView( model.getClock(), model, mvt );
    beakerFrontLayer.addChild( beakerView.getFrontNode() );
    beakerBackLayer.addChild( beakerView.getBackNode() );
    beakerGrabLayer.addChild( beakerView.getGrabNode() );
    // Add the thermometer nodes.
    var movableThermometerNodes = [];
    for ( var thermometer in model.thermometers ) {
      var thermometerNode = new MovableThermometerNode( thermometer, mvt );
      thermometerLayer.addChild( thermometerNode );
      thermometerNode.addInputEventListener( new PBasicInputEventHandler().withAnonymousClassBody( {
        mouseReleased: function( event ) {
          if ( thermometerNode.getFullBoundsReference().intersects( thermometerToolBox.getFullBoundsReference() ) ) {
            // Released over tool box, so deactivate.
            thermometer.active.set( false );
          }
        }
      } ) );
      movableThermometerNodes.add( thermometerNode );
    }
    // Add the tool box for the thermometers.
    var thermometerBox = new HBox();
    var thermometerToolBoxNodes = [];
    for ( var movableThermometerNode in movableThermometerNodes ) {
      var thermometerToolBoxNode = new ThermometerToolBoxNode( movableThermometerNode, mvt, this );
      thermometerBox.addChild( thermometerToolBoxNode );
      thermometerToolBoxNodes.add( thermometerToolBoxNode );
    }
    thermometerToolBox = new ControlPanelNode( thermometerBox, CONTROL_PANEL_BACKGROUND_COLOR, CONTROL_PANEL_OUTLINE_STROKE, CONTROL_PANEL_OUTLINE_COLOR );
    thermometerToolBox.setOffset( EDGE_INSET, EDGE_INSET );
    backLayer.addChild( thermometerToolBox );
    for ( var thermometerToolBoxNode in thermometerToolBoxNodes ) {
      thermometerToolBoxNode.setReturnRect( thermometerBox.getFullBoundsReference() );
    }
    // user controlled state changes.
    var blockChangeObserver = new SimpleObserver().withAnonymousClassBody( {
      update: function() {
        if ( model.getIronBlock().isStackedUpon( model.getBrick() ) ) {
          brickNode.moveToBack();
        }
        else if ( model.getBrick().isStackedUpon( model.getIronBlock() ) ) {
          ironBlockNode.moveToBack();
        }
        else if ( model.getIronBlock().getRect().getMinX() >= model.getBrick().getRect().getMaxX() || model.getIronBlock().getRect().getMinY() >= model.getBrick().getRect().getMaxY() ) {
          ironBlockNode.moveToFront();
        }
        else if ( model.getBrick().getRect().getMinX() >= model.getIronBlock().getRect().getMaxX() || model.getBrick().getRect().getMinY() >= model.getIronBlock().getRect().getMaxY() ) {
          brickNode.moveToFront();
        }
      }
    } );
    // blocks can be extracted.
    model.getBeaker().fluidLevel.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( fluidLevel ) {
        if ( fluidLevel != Beaker.INITIAL_FLUID_LEVEL ) {
          beakerGrabLayer.moveInBackOf( blockLayer );
        }
        else {
          beakerGrabLayer.moveInFrontOf( blockLayer );
        }
      }
    } ) );
    // of either changes.
    model.getBrick().position.addObserver( blockChangeObserver );
    model.getIronBlock().position.addObserver( blockChangeObserver );
  }

  return inherit( PhetPCanvas, EFACIntroCanvas, {
      reset: function() {
        model.reset();
        normalSimSpeed.reset();
        model.getClock().setPaused( false );
      }
    },
//statics
    {
      STAGE_SIZE: STAGE_SIZE,
      BURNER_EDGE_TO_HEIGHT_RATIO: BURNER_EDGE_TO_HEIGHT_RATIO,
      showDumpEnergiesButton: showDumpEnergiesButton
    } );
} );

