// Copyright 2002-2013, University of Colorado
/**
 * Thermometer node that the user can drag around and that updates its
 * temperature reading based on the reading from the supplied model element.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Rectangle = require( 'KITE/Rectangle' );
  var MathUtil = require( 'edu.colorado.phet.common.phetcommon.math.MathUtil' );
  var Vector2 = require( 'DOT/Vector2' );
  var Function1 = require( 'edu.colorado.phet.common.phetcommon.util.function.Function1' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var Thermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Thermometer' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PDimension = require( 'edu.umd.cs.piccolo.util.PDimension' );


  // static class: ThermometerLocationConstraint
  var ThermometerLocationConstraint =
// Class that constrains the valid locations for a thermometer.

    //private
    define( function( require ) {

      //private
      function ThermometerLocationConstraint( mvt, node, offsetPosToNodeCenter ) {

        //private
        this.modelBounds;
        var nodeSize = new PDimension( node.getFullBoundsReference().width, node.getFullBoundsReference().height );
        // the nature of the provided node.
        var boundsMinX = mvt.viewToModelX( nodeSize.getWidth() / 2 - offsetPosToNodeCenter.getX() );
        var boundsMaxX = mvt.viewToModelX( EFACIntroCanvas.STAGE_SIZE.getWidth() - nodeSize.getWidth() / 2 - offsetPosToNodeCenter.getX() );
        var boundsMinY = mvt.viewToModelY( EFACIntroCanvas.STAGE_SIZE.getHeight() - offsetPosToNodeCenter.getY() - nodeSize.getHeight() / 2 );
        var boundsMaxY = mvt.viewToModelY( -offsetPosToNodeCenter.getY() + nodeSize.getHeight() / 2 );
        modelBounds = new Rectangle.Number( boundsMinX, boundsMinY, boundsMaxX - boundsMinX, boundsMaxY - boundsMinY );
      }

      return inherit( Object, ThermometerLocationConstraint, {
        apply: function( proposedModelPos ) {
          var constrainedXPos = MathUtil.clamp( modelBounds.getMinX(), proposedModelPos.getX(), modelBounds.getMaxX() );
          var constrainedYPos = MathUtil.clamp( modelBounds.getMinY(), proposedModelPos.getY(), modelBounds.getMaxY() );
          return new Vector2( constrainedXPos, constrainedYPos );
        }
      } );
    } );
  ;
  function MovableThermometerNode( thermometer, mvt ) {
    SensingThermometerNode.call( this, thermometer );
    // Update the offset if and when the model position changes.
    thermometer.position.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( position ) {
        setOffset( mvt.modelToViewX( position.getX() ), mvt.modelToViewY( position.getY() ) - (getFullBoundsReference().height / 2 + triangleTipOffset.getHeight()) );
      }
    } ) );
    // Add the drag handler.
    var offsetPosToCenter = new Vector2( getFullBoundsReference().getCenterX() - mvt.modelToViewX( thermometer.position.get().getX() ), getFullBoundsReference().getCenterY() - mvt.modelToViewY( thermometer.position.get().getY() ) );
    addInputEventListener( new ThermalElementDragHandler( thermometer, this, mvt, new ThermometerLocationConstraint( mvt, this, offsetPosToCenter ) ) );
  }

  return inherit( SensingThermometerNode, MovableThermometerNode, {
  } );
} );

