// Copyright 2002-2012, University of Colorado
/**
 * Clock control panel that shows "Normal Speed" and "Fast Forward" as radio
 * buttons with a play/pause and step button.
 *
 * @author Sam Reid
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Color = require( 'SCENERY/util/Color' );
  var IClock = require( 'edu.colorado.phet.common.phetcommon.model.clock.IClock' );
  var SettableProperty = require( 'edu.colorado.phet.common.phetcommon.model.property.SettableProperty' );
  var PropertyRadioButton = require( 'edu.colorado.phet.common.phetcommon.view.controls.PropertyRadioButton' );
  var PhetFont = require( 'edu.colorado.phet.common.phetcommon.view.util.PhetFont' );
  var RichPNode = require( 'edu.colorado.phet.common.piccolophet.RichPNode' );
  var HBox = require( 'edu.colorado.phet.common.piccolophet.nodes.layout.HBox' );
  var PiccoloClockControlPanel = require( 'edu.colorado.phet.common.piccolophet.nodes.mediabuttons.PiccoloClockControlPanel' );
  var EnergyFormsAndChangesResources = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesResources' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PSwing = require( 'edu.umd.cs.piccolox.pswing.PSwing' );
  var fastForwardRadioButton = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing/UserComponents/fastForwardRadioButton' );//static
  var normalMotionRadioButton = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/EnergyFormsAndChangesSimSharing/UserComponents/normalMotionRadioButton' );//static

  /*
   * Constructor.
   */
  function NormalAndFastForwardTimeControlPanel( normalSpeed, clock ) {

    //private
    this.TRANSPARENT = new Color( 0, 0, 0, 0 );

    //private
    this.RADIO_BUTTON_FONT = new PhetFont( 16 );
    var clockControlPanel = new PSwing( new PiccoloClockControlPanel( clock ).withAnonymousClassBody( {
      initializer: function() {
        setBackground( TRANSPARENT );
        getButtonCanvas().setBackground( TRANSPARENT );
        getBackgroundNode().setVisible( false );
        clearAllToolTips();
      }
    } ) );
    var normalSpeedButton = new PSwing( new PropertyRadioButton( normalMotionRadioButton, EnergyFormsAndChangesResources.Strings.NORMAL, normalSpeed, true ).withAnonymousClassBody( {
      initializer: function() {
        setBackground( TRANSPARENT );
        setFont( RADIO_BUTTON_FONT );
      }
    } ) );
    var fastForwardButton = new PSwing( new PropertyRadioButton( fastForwardRadioButton, EnergyFormsAndChangesResources.Strings.FAST_FORWARD, normalSpeed, false ).withAnonymousClassBody( {
      initializer: function() {
        setBackground( TRANSPARENT );
        setFont( RADIO_BUTTON_FONT );
      }
    } ) );
    addChild( new HBox( normalSpeedButton, fastForwardButton, clockControlPanel ) );
  }

  return inherit( RichPNode, NormalAndFastForwardTimeControlPanel, {
  } );
} );

