// Copyright 2002-2013, University of Colorado
/**
 * Thermometer node that updates its displayed temperature and color based on
 * what is being "sensed" by the supplied thermometer model element.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var Thermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Thermometer' );

  function SensingThermometerNode( thermometer ) {

    //private
    this.thermometer;
    this.thermometer = thermometer;
    thermometer.sensedTemperature.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( sensedTemperature ) {
        setSensedTemperature( sensedTemperature );
      }
    } ) );
    thermometer.sensedElementColor.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( sensedColor ) {
        setSensedColor( sensedColor );
      }
    } ) );
    thermometer.active.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( active ) {
        setVisible( active );
      }
    } ) );
  }

  return inherit( ThermometerNode, SensingThermometerNode, {
    getThermometer: function() {
      return thermometer;
    }
  } );
} );

