// Copyright 2002-2012, University of Colorado
/**
 * Drag handler for objects that can be moved by the user.  This is constructed
 * with a constraint function that defines where the model object can go.
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Function1 = require( 'edu.colorado.phet.common.phetcommon.util.function.Function1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var RelativeDragHandler = require( 'edu.colorado.phet.common.piccolophet.event.RelativeDragHandler' );
  var UserMovableModelElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/UserMovableModelElement' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PInputEvent = require( 'edu.umd.cs.piccolo.event.PInputEvent' );

  /*
   * Constructor.  The node must be property positioned before calling
   * this, or it won't work correctly.
   */
  function ThermalElementDragHandler( modelElement, node, mvt, constraint ) {

    //private
    this.modelElement;
    RelativeDragHandler.call( this, node, mvt, modelElement.position, constraint );
    this.modelElement = modelElement;
  }

  return inherit( RelativeDragHandler, ThermalElementDragHandler, {
    mousePressed: function( event ) {
      super.mousePressed( event );
      modelElement.userControlled.set( true );
    },
    mouseReleased: function( event ) {
      super.mouseReleased( event );
      modelElement.userControlled.set( false );
    }
  } );
} );

