// Copyright 2002-2012, University of Colorado
/**
 * This class defines the motion constraints for the model elements that can
 * be moved on and off the burners.  These elements are prevented from
 * overlapping one another.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Rectangle = require( 'KITE/Rectangle' );
  var MathUtil = require( 'edu.colorado.phet.common.phetcommon.math.MathUtil' );
  var Vector2 = require( 'DOT/Vector2' );
  var Function1 = require( 'edu.colorado.phet.common.phetcommon.util.function.Function1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var EFACIntroModel = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/EFACIntroModel' );
  var RectangularThermalMovableModelElement = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/RectangularThermalMovableModelElement' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PDimension = require( 'edu.umd.cs.piccolo.util.PDimension' );

  function ThermalItemMotionConstraint( model, modelElement, node, mvt, offsetPosToNodeCenter ) {

    //private
    this.modelBounds;

    //private
    this.model;

    //private
    this.modelElement;
    this.model = model;
    this.modelElement = modelElement;
    var nodeSize = new PDimension( node.getFullBoundsReference().width, node.getFullBoundsReference().height );
    // the nature of the provided node.
    var boundsMinX = mvt.viewToModelX( nodeSize.getWidth() / 2 - offsetPosToNodeCenter.getX() );
    var boundsMaxX = mvt.viewToModelX( EFACIntroCanvas.STAGE_SIZE.getWidth() - nodeSize.getWidth() / 2 - offsetPosToNodeCenter.getX() );
    var boundsMinY = mvt.viewToModelY( EFACIntroCanvas.STAGE_SIZE.getHeight() - offsetPosToNodeCenter.getY() - nodeSize.getHeight() / 2 );
    var boundsMaxY = mvt.viewToModelY( -offsetPosToNodeCenter.getY() + nodeSize.getHeight() / 2 );
    modelBounds = new Rectangle.Number( boundsMinX, boundsMinY, boundsMaxX - boundsMinX, boundsMaxY - boundsMinY );
  }

  return inherit( Object, ThermalItemMotionConstraint, {
    apply: function( proposedModelPos ) {
      // Make sure the proposed position is on the stage.
      var constrainedXPos = MathUtil.clamp( modelBounds.getMinX(), proposedModelPos.getX(), modelBounds.getMaxX() );
      var constrainedYPos = MathUtil.clamp( modelBounds.getMinY(), proposedModelPos.getY(), modelBounds.getMaxY() );
      var boundedToStagePos = new Vector2( constrainedXPos, constrainedYPos );
      // Return a position that is also validated by the model.
      return model.validatePosition( modelElement, boundedToStagePos );
    }
  } );
} );

