// Copyright 2002-2013, University of Colorado
/**
 * PNode that is used to represent thermometers in the tool box and that
 * controls the initial movement of thermometers in and out of the tool
 * box.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var Rectangle = require( 'KITE/Rectangle' );
  var Vector2 = require( 'DOT/Vector2' );
  var VoidFunction1 = require( 'edu.colorado.phet.common.phetcommon.util.function.VoidFunction1' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPCanvas = require( 'edu.colorado.phet.common.piccolophet.PhetPCanvas' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var Thermometer = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/Thermometer' );
  var PBasicInputEventHandler = require( 'edu.umd.cs.piccolo.event.PBasicInputEventHandler' );
  var PInputEvent = require( 'edu.umd.cs.piccolo.event.PInputEvent' );

  function ThermometerToolBoxNode( thermometerNode, mvt, canvas ) {

    //private
    this.canvas;

    //private
    this.mvt;

    //private
    this.returnRect = null;
    this.canvas = canvas;
    this.mvt = mvt;
    var thermometer = thermometerNode.getThermometer();
    var positioningOffset = mvt.viewToModelDelta( thermometerNode.getOffsetCenterShaftToTriangleTip() );
    setSensedTemperature( EFACConstants.ROOM_TEMPERATURE );
    setSensedColor( Color.WHITE );
    // This node's visibility is the inverse of the thermometer's.
    thermometer.active.addObserver( new VoidFunction1().withAnonymousClassBody( {
      apply: function( active ) {
        setVisible( !active );
      }
    } ) );
    addInputEventListener( new PBasicInputEventHandler().withAnonymousClassBody( {
      mousePressed: function( event ) {
        thermometer.active.set( true );
        thermometer.userControlled.set( true );
        thermometer.position.set( new Vector2( getModelPosition( event.getCanvasPosition() ) ).plus( positioningOffset ) );
      },
      mouseDragged: function( event ) {
        thermometer.position.set( new Vector2( getModelPosition( event.getCanvasPosition() ) ).plus( positioningOffset ) );
      },
      mouseReleased: function( event ) {
        thermometer.userControlled.set( false );
        if ( returnRect != null && thermometerNode.getFullBoundsReference().intersects( returnRect ) ) {
          // Released over tool box, so return it.
          thermometer.active.set( false );
        }
      }
    } ) );
  }

  return inherit( ThermometerNode, ThermometerToolBoxNode, {
    /**
     * Convert the canvas position to the corresponding location in the model.
     */

    //private
    getModelPosition: function( canvasPos ) {
      var worldPos = new Vector2( canvasPos.getX(), canvasPos.getY() );
      canvas.getPhetRootNode().screenToWorld( worldPos );
      var adjustedWorldPos = new Vector2( worldPos.getX(), worldPos.getY() );
      return mvt.viewToModel( adjustedWorldPos );
    },
    setReturnRect: function( returnRect ) {
      this.returnRect = returnRect;
    }
  } );
} );

