// Copyright 2002-2013, University of Colorado
/**
 * Test bed for energy chunk distribution algorithms.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var inherit = require( 'PHET_CORE/inherit' );
  var ActionEvent = require( 'java.awt.event.ActionEvent' );
  var ActionListener = require( 'java.awt.event.ActionListener' );
  var Dimension2D = require( 'java.awt.geom.Dimension2D' );
  var Ellipse2D = require( 'java.awt.geom.Ellipse2D' );
  var Vector2 = require( 'java.awt.geom.Vector2' );
  var ArrayList = require( 'java.util.ArrayList' );
  var List = require( 'java.util.List' );
  var swing = require( 'javax.swing' );//.*
  var Vector2 = require( 'DOT/Vector2' );
  var Resettable = require( 'edu.colorado.phet.common.phetcommon.model.Resettable' );
  var Property = require( 'AXON/Property' );
  var Property = require( 'AXON/Property' );
  var ModelViewTransform = require( 'edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform' );
  var PhetPCanvas = require( 'edu.colorado.phet.common.piccolophet.PhetPCanvas' );
  var ResetAllButtonNode = require( 'edu.colorado.phet.common.piccolophet.nodes.ResetAllButtonNode' );
  var EFACConstants = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/EFACConstants' );
  var EnergyChunk = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyChunk' );
  var EnergyType = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/model/EnergyType' );
  var EnergyChunkNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/common/view/EnergyChunkNode' );
  var EnergyChunkContainerSlice = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/EnergyChunkContainerSlice' );
  var EnergyChunkContainerSliceNode = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/EnergyChunkContainerSliceNode' );
  var EnergyChunkDistributor = require( 'ENERGY_FORMS_AND_CHANGES/energy-forms-and-changes/intro/model/EnergyChunkDistributor' );
  var Node = require( 'SCENERY/nodes/Node' );
  var PDimension = require( 'edu.umd.cs.piccolo.util.PDimension' );


  //private
  var NUM_ENERGY_CHUNKS = 50;
// In meters.

  //private
  var CONTAINER_WIDTH = 0.05;

  return inherit( Object, TestEnergyChunkDistribution, {
    /*
     * Main routine that constructs a PhET Piccolo canvas in a window.
     */
    main: function( args ) {
      var stageSize = new PDimension( 500, 400 );
      var canvas = new PhetPCanvas();
      // Set up the canvas-screen transform.
      canvas.setWorldTransformStrategy( new PhetPCanvas.CenteredStage( canvas, stageSize ) );
      // Set up the model-view transform.
      var mvt = ModelViewTransform.createSinglePointScaleInvertedYMapping( new Vector2( 0, 0 ), new Point( Math.round( stageSize.getWidth() * 0.5 ), Math.round( stageSize.getHeight() * 0.50 ) ), // "Zoom factor" - smaller zooms out, larger zooms in.
        4000 );
      // Root node for adding other nodes.
      var rootNode = new Node();
      canvas.getLayer().addChild( rootNode );
      //        Shape containerShape = new Rectangle2D.Double( -CONTAINER_WIDTH / 2, -CONTAINER_WIDTH / 2, CONTAINER_WIDTH, CONTAINER_WIDTH );
      var containerShape = new Ellipse2D.Number( -CONTAINER_WIDTH / 2, -CONTAINER_WIDTH / 2 * 2, CONTAINER_WIDTH, CONTAINER_WIDTH * 2 );
      var energyChunkContainerSlice = new EnergyChunkContainerSlice( containerShape, 0, new Property( new Vector2( 0, 0 ) ) );
      var sliceNode = new EnergyChunkContainerSliceNode( energyChunkContainerSlice, mvt, true, Color.BLUE );
      rootNode.addChild( sliceNode );
      // Add the energy chunks.
      for ( var i = 0; i < NUM_ENERGY_CHUNKS; i++ ) {
        var energyChunk = new EnergyChunk( EnergyType.THERMAL, EnergyChunkDistributor.generateRandomLocation( energyChunkContainerSlice.getShape().getBounds2D() ), new BooleanProperty( true ) );
        energyChunkContainerSlice.addEnergyChunk( energyChunk );
        rootNode.addChild( new EnergyChunkNode( energyChunk, mvt ) );
      }
      // Set up a timer to update the state.
      var dt = 1 / EFACConstants.FRAMES_PER_SECOND;
      var sliceList = [];
      sliceList.add( energyChunkContainerSlice );
      var timer = new Timer( Math.round( dt * 1000 ), new ActionListener().withAnonymousClassBody( {
        actionPerformed: function( e ) {
          EnergyChunkDistributor.updatePositions( sliceList, dt );
        }
      } ) );
      timer.start();
      // Add a button that will reset the state.
      var resetButton = new ResetAllButtonNode( new Resettable().withAnonymousClassBody( {
        reset: function() {
          for ( var ec in energyChunkContainerSlice.energyChunkList ) {
            ec.setVelocity( 0, 0 );
          }
          randomizeEnergyChunkPositions( energyChunkContainerSlice );
        }
      } ), canvas, 16, Color.BLACK, Color.ORANGE );
      resetButton.setConfirmationEnabled( false );
      resetButton.setOffset( sliceNode.getFullBoundsReference().getCenterX(), sliceNode.getFullBoundsReference().getMaxY() + 20 );
      rootNode.addChild( resetButton );
      // Boiler plate app stuff.
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.setSize( stageSize.getWidth(), stageSize.getHeight() );
      frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
      // Center.
      frame.setLocationRelativeTo( null );
      frame.setVisible( true );
    },

    //private
    randomizeEnergyChunkPositions: function( energyChunkContainerSlice ) {
      for ( var ec in energyChunkContainerSlice.energyChunkList ) {
        ec.position.set( EnergyChunkDistributor.generateRandomLocation( energyChunkContainerSlice.getShape().getBounds2D() ) );
      }
    }
  } );
} );

